@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 #--------------------------------------------------------------------------#
 
 set -e -o pipefail
@@ -24,6 +24,7 @@ General options;
   --gpl                    permit GPL dependencies, if available
   --arm64                  cross-compile for Linux ARM 64 bit
   --win64                  cross-compile for Windows 64 bit
+  --win64-native           natively compile for Windows 64 bit
   --ninja                  use Ninja build system
   --docs                   build Api documentation
 
@@ -132,6 +133,7 @@ ubsan=default
 unit_testing=default
 valgrind=default
 win64=default
+win64_native=default
 arm64=default
 werror=default
 ipo=default
@@ -217,6 +219,8 @@ do
 
     --win64) win64=ON;;
 
+    --win64-native) win64_native=ON;;
+
     --arm64) arm64=ON;;
 
     --ninja) ninja=ON;;
@@ -336,6 +340,9 @@ fi
   && cmake_opts="$cmake_opts -DENABLE_GPL=$gpl"
 [ $win64 != default ] \
   && cmake_opts="$cmake_opts -DCMAKE_TOOLCHAIN_FILE=../cmake/Toolchain-mingw64.cmake"
+# Because 'MSYS Makefiles' has a space in it, we set the variable vs. adding to 'cmake_opts'
+[ $win64_native != default ] \
+  && [ $ninja == default ] && export CMAKE_GENERATOR="MSYS Makefiles"
 [ $arm64 != default ] \
   && cmake_opts="$cmake_opts -DCMAKE_TOOLCHAIN_FILE=../cmake/Toolchain-aarch64.cmake"
 [ $ninja != default ] && cmake_opts="$cmake_opts -G Ninja"
