@@ -30,8 +30,9 @@
 #include "expr/skolem_manager.h"
 #include "expr/type_checker.h"
 #include "expr/type_properties.h"
+#include "theory/builtin/abstract_type.h"
 #include "util/bitvector.h"
-#include "util/ff_val.h"
+#include "util/finite_field_value.h"
 #include "util/integer.h"
 #include "util/poly_util.h"
 #include "util/rational.h"
@@ -482,7 +483,7 @@ std::vector<NodeValue*> NodeManager::TopologicalSort(
   return order;
 } /* NodeManager::TopologicalSort() */
 
-TypeNode NodeManager::getType(TNode n, bool check)
+TypeNode NodeManager::getType(TNode n, bool check, std::ostream* errOut)
 {
   TypeNode typeNode;
   bool hasType = getAttribute(n, TypeAttr(), typeNode);
@@ -526,7 +527,7 @@ TypeNode NodeManager::getType(TNode n, bool check)
       {
         Assert(check || m.getMetaKind() != kind::metakind::NULLARY_OPERATOR);
         /* All the children have types, time to compute */
-        typeNode = TypeChecker::computeType(this, m, check);
+        typeNode = TypeChecker::computeType(this, m, check, errOut);
         worklist.pop();
       }
     }  // end while
@@ -565,6 +566,50 @@ TypeNode NodeManager::mkSequenceType(TypeNode elementType)
   return mkTypeNode(kind::SEQUENCE_TYPE, elementType);
 }
 
+bool NodeManager::isSortKindAbstractable(Kind k)
+{
+  return k == kind::ABSTRACT_TYPE || k == kind::ARRAY_TYPE
+         || k == kind::BAG_TYPE || k == kind::BITVECTOR_TYPE
+         || k == kind::TUPLE_TYPE || k == kind::FINITE_FIELD_TYPE
+         || k == kind::FLOATINGPOINT_TYPE || k == kind::FUNCTION_TYPE
+         || k == kind::SEQUENCE_TYPE || k == kind::SET_TYPE;
+}
+
+TypeNode NodeManager::mkAbstractType(Kind k)
+{
+  if (!isSortKindAbstractable(k))
+  {
+    std::stringstream ss;
+    ss << "Cannot construct abstract type for kind " << k;
+    throw Exception(ss.str());
+  }
+  if (k == kind::ARRAY_TYPE)
+  {
+    // ?Array -> (Array ? ?)
+    TypeNode a = mkAbstractType(kind::ABSTRACT_TYPE);
+    return mkArrayType(a, a);
+  }
+  if (k == kind::SET_TYPE)
+  {
+    // ?Set -> (Set ?)
+    TypeNode a = mkAbstractType(kind::ABSTRACT_TYPE);
+    return mkSetType(a);
+  }
+  if (k == kind::BAG_TYPE)
+  {
+    // ?Bag -> (Bag ?)
+    TypeNode a = mkAbstractType(kind::ABSTRACT_TYPE);
+    return mkBagType(a);
+  }
+  if (k == kind::SEQUENCE_TYPE)
+  {
+    // ?Seq -> (Seq ?)
+    TypeNode a = mkAbstractType(kind::ABSTRACT_TYPE);
+    return mkSequenceType(a);
+  }
+  return mkTypeConst<AbstractType>(AbstractType(k));
+}
+
 TypeNode NodeManager::mkDatatypeType(DType& datatype)
 {
   // Not worth a special implementation; this doesn't need to be fast
