@@ -335,8 +335,9 @@ class NodeManager
    * @param n the Node for which we want a type
    * @param check whether we should check the type as we compute it
    * (default: false)
+   * @param errOut An (optional) output stream to print type checking errors
    */
-  TypeNode getType(TNode n, bool check = false);
+  TypeNode getType(TNode n, bool check = false, std::ostream* errOut = nullptr);
 
   /** Get the (singleton) type for Booleans. */
   TypeNode booleanType();
@@ -457,6 +458,15 @@ class NodeManager
   /** Make the type of sequences with the given parameterization */
   TypeNode mkSequenceType(TypeNode elementType);
 
+  /**
+   * @return True if `k` is an abstractable sort kind, i.e., a valid argument to
+   * `mkAbstractType`.
+   */
+  static bool isSortKindAbstractable(Kind k);
+
+  /** Make the abstract type with the given kind */
+  TypeNode mkAbstractType(Kind k);
+
   /** Make a type representing the given datatype. */
   TypeNode mkDatatypeType(DType& datatype);
 
