@@ -22,11 +22,12 @@
 #include "expr/type_properties.h"
 #include "options/base_options.h"
 #include "options/quantifiers_options.h"
+#include "theory/builtin/abstract_type.h"
 #include "theory/fp/theory_fp_utils.h"
 #include "theory/type_enumerator.h"
 #include "util/bitvector.h"
 #include "util/cardinality.h"
-#include "util/ff_val.h"
+#include "util/finite_field_value.h"
 #include "util/integer.h"
 
 using namespace std;
@@ -291,9 +292,11 @@ bool TypeNode::isClosedEnumerable()
 
 bool TypeNode::isFirstClass() const
 {
-  return getKind() != kind::CONSTRUCTOR_TYPE && getKind() != kind::SELECTOR_TYPE
-         && getKind() != kind::TESTER_TYPE && getKind() != kind::UPDATER_TYPE
-         && (getKind() != kind::TYPE_CONSTANT
+  Kind k = getKind();
+  return k != kind::CONSTRUCTOR_TYPE && k != kind::SELECTOR_TYPE
+         && k != kind::TESTER_TYPE && k != kind::UPDATER_TYPE
+         && k != kind::ABSTRACT_TYPE
+         && (k != kind::TYPE_CONSTANT
              || (getConst<TypeConstant>() != REGEXP_TYPE
                  && getConst<TypeConstant>() != SEXPR_TYPE));
 }
@@ -316,6 +319,89 @@ bool TypeNode::isReal() const
 
 bool TypeNode::isStringLike() const { return isString() || isSequence(); }
 
+bool TypeNode::isInstanceOf(const TypeNode& t) const
+{
+  return leastUpperBound(t) == (*this);
+}
+
+TypeNode TypeNode::leastUpperBound(const TypeNode& t) const
+{
+  return unifyInternal(t, true);
+}
+
+TypeNode TypeNode::greatestLowerBound(const TypeNode& t) const
+{
+  return unifyInternal(t, false);
+}
+
+TypeNode TypeNode::unifyInternal(const TypeNode& t, bool isLub) const
+{
+  Assert(!isNull() && !t.isNull());
+  if (*this == t)
+  {
+    return t;
+  }
+  if (t.isAbstract())
+  {
+    Kind tak = t.getAbstractedKind();
+    if (tak == kind::ABSTRACT_TYPE)
+    {
+      // everything is unifiable with the fully abstract type
+      return isLub ? *this : t;
+    }
+    // ABSTRACT_TYPE{k} is unifiable with types with kind k
+    if (getKind() == tak)
+    {
+      return isLub ? *this : t;
+    }
+  }
+  // same as above, swapping this and t
+  if (isAbstract())
+  {
+    Kind ak = getAbstractedKind();
+    if (ak == kind::ABSTRACT_TYPE)
+    {
+      return isLub ? t : *this;
+    }
+    if (t.getKind() == ak)
+    {
+      return isLub ? t : *this;
+    }
+  }
+  Kind k = getKind();
+  if (k == kind::TYPE_CONSTANT || k != t.getKind())
+  {
+    // different kinds, or distinct constants
+    return TypeNode::null();
+  }
+  size_t nchild = getNumChildren();
+  if (nchild == 0 || nchild != t.getNumChildren())
+  {
+    // different arities
+    return TypeNode::null();
+  }
+  NodeBuilder nb(k);
+  for (size_t i = 0; i < nchild; i++)
+  {
+    TypeNode c = (*this)[i];
+    TypeNode tc = t[i];
+    TypeNode jc = c.unifyInternal(tc, isLub);
+    if (jc.isNull())
+    {
+      // incompatible component type
+      return jc;
+    }
+    nb << jc;
+  }
+  return nb.constructTypeNode();
+}
+
+bool TypeNode::isComparableTo(const TypeNode& t) const
+{
+  // could do join or meet here
+  return !unifyInternal(t, true).isNull();
+}
+
 bool TypeNode::isRealOrInt() const { return isReal() || isInteger(); }
 
 TypeNode TypeNode::getDatatypeTesterDomainType() const
@@ -558,6 +644,36 @@ bool TypeNode::isSygusDatatype() const
   return false;
 }
 
+bool TypeNode::isAbstract() const { return getKind() == kind::ABSTRACT_TYPE; }
+
+bool TypeNode::isFullyAbstract() const
+{
+  return getKind() == kind::ABSTRACT_TYPE
+         && getAbstractedKind() == kind::ABSTRACT_TYPE;
+}
+
+Kind TypeNode::getAbstractedKind() const
+{
+  Assert(isAbstract());
+  const AbstractType& ak = getConst<AbstractType>();
+  return ak.getKind();
+}
+
+bool TypeNode::isMaybeKind(Kind k) const
+{
+  Kind tk = getKind();
+  if (tk == k)
+  {
+    return true;
+  }
+  if (tk == kind::ABSTRACT_TYPE)
+  {
+    Kind tak = getAbstractedKind();
+    return tak == kind::ABSTRACT_TYPE || tak == k;
+  }
+  return false;
+}
+
 std::string TypeNode::toString() const {
   std::stringstream ss;
   toStream(ss);
