@@ -477,6 +477,12 @@ class CVC5_EXPORT TypeNode
   /** Is this a Sequence type? */
   bool isSequence() const;
 
+  /** Is this an abstract type? */
+  bool isAbstract() const;
+
+  /** Is this the fully abstract type? */
+  bool isFullyAbstract() const;
+
   /** Get the index type (for array types) */
   TypeNode getArrayIndexType() const;
 
@@ -503,6 +509,25 @@ class CVC5_EXPORT TypeNode
 
   /** Get the element type (for sequence types) */
   TypeNode getSequenceElementType() const;
+
+  /** Get the abstract kind (for abstract types) */
+  Kind getAbstractedKind() const;
+
+  /**
+   * Is maybe kind. Return true if an instance of this type may have kind k.
+   * This is true if the kind of this sort is k, or if it is a abstract type
+   * whose abstracted kind is k or ABSTRACT_TYPE (the fully abstract type).
+   *
+   * For example:
+   * isMaybeKind ? BITVECTOR_TYPE = true
+   * isMaybeKind ? SET_TYPE = true
+   * isMaybeKind ?Set SET_TYPE = true
+   * isMaybeKind (Set Int) SET_TYPE = true
+   * isMaybeKind (_ BitVec 4) SET_TYPE = false
+   * isMaybeKind ?BitVec SET_TYPE = false
+   */
+  bool isMaybeKind(Kind k) const;
+
   /**
    * Is this a function type?  Function-like things (e.g. datatype
    * selectors) that aren't actually functions are NOT considered
@@ -523,6 +548,38 @@ class CVC5_EXPORT TypeNode
    */
   bool isFunctionLike() const;
 
+  /**
+   * Is instance of, returns true if this type is equivalent to the
+   * leastUpperBound (see TypeNode::leastUpperBound) of itself and t.
+   */
+  bool isInstanceOf(const TypeNode& t) const;
+  /**
+   * Is comparable to type t, returns true if this type and t have a non-null
+   * leastUpperBound (see TypeNode::leastUpperBound).
+   */
+  bool isComparableTo(const TypeNode& t) const;
+  /**
+   * Least upper bound with type.
+   *
+   * We consider a partial order on types such that T1 <= T2 if T2 is an
+   * instance of T1.
+   *
+   * This returns the most specific type that is an instance
+   * of both this and t, or null if this type and t are incompatible.
+   *
+   * For example:
+   * ?BitVec <lub> ? = ?BitVec
+   * (Array ?BitVec Int) <lub> (Array (_ BitVec 4) ?) = (Array (_ BitVec 4) Int)
+   * (Array ? Int) <lub> (Array ? Real) = null.
+   */
+  TypeNode leastUpperBound(const TypeNode& t) const;
+  /**
+   * Greatest lower bound with type. The dual of leastUpperBound, for example:
+   * ?BitVec <glb> ? = ?
+   * (Array ?BitVec Int) <glb> (Array (_ BitVec 4) ?) = (Array ?BitVec ?)
+   * (Array ? Int) <glb> (Array ? Real) = null.
+   */
+  TypeNode greatestLowerBound(const TypeNode& t) const;
   /**
    * Get the argument types of a function, datatype constructor,
    * datatype selector, or datatype tester.
@@ -699,20 +756,9 @@ class CVC5_EXPORT TypeNode
    */
   TypeNode getUninterpretedSortConstructor() const;
 
-private:
-
-  /**
-   * Indents the given stream a given amount of spaces.
-   *
-   * @param out the stream to indent
-   * @param indent the number of spaces
-   */
-  static void indent(std::ostream& out, int indent) {
-    for(int i = 0; i < indent; i++) {
-      out << ' ';
-    }
-  }
-
+ private:
+  /** Unify internal, for computing leastUpperBound and greatestLowerBound */
+  TypeNode unifyInternal(const TypeNode& t, bool isLub) const;
 };/* class TypeNode */
 
 /**
