@@ -39,12 +39,14 @@
 #  endif /* HAVE_EXT_STDIO_FILEBUF_H */
 #endif   /* HAVE_LIBEDITLINE */
 
-#include "api/cpp/cvc5.h"
+#include <cvc5/cvc5.h>
+
 #include "base/check.h"
 #include "base/output.h"
+#include "main/command_executor.h"
 #include "parser/api/cpp/command.h"
+#include "parser/api/cpp/input_parser.h"
 #include "parser/api/cpp/symbol_manager.h"
-#include "parser/input_parser.h"
 #include "theory/logic_info.h"
 
 using namespace std;
@@ -80,12 +82,13 @@ static set<string> s_declarations;
 
 #endif /* HAVE_LIBEDITLINE */
 
-InteractiveShell::InteractiveShell(Solver* solver,
-                                   SymbolManager* sm,
+InteractiveShell::InteractiveShell(main::CommandExecutor* cexec,
                                    std::istream& in,
                                    std::ostream& out,
                                    bool isInteractive)
-    : d_solver(solver),
+    : d_cexec(cexec),
+      d_solver(cexec->getSolver()),
+      d_symman(cexec->getSymbolManager()),
       d_in(in),
       d_out(out),
       d_isInteractive(isInteractive),
@@ -94,11 +97,13 @@ InteractiveShell::InteractiveShell(Solver* solver,
   if (d_solver->getOptionInfo("force-logic").setByUser)
   {
     LogicInfo tmp(d_solver->getOption("force-logic"));
-    sm->forceLogic(tmp.getLogicString());
+    d_symman->forceLogic(tmp.getLogicString());
   }
   /* Create parser with bogus input. */
-  d_parser.reset(new cvc5::parser::InputParser(solver, sm, true));
-
+  d_parser.reset(new cvc5::parser::InputParser(d_solver, d_symman));
+  // initialize for incremental string input
+  d_parser->setIncrementalStringInput(d_solver->getOption("input-language"),
+                                      INPUT_FILENAME);
 #if HAVE_LIBEDITLINE
   if (&d_in == &std::cin && isatty(fileno(stdin)))
   {
@@ -110,7 +115,7 @@ InteractiveShell::InteractiveShell(Solver* solver,
 #endif /* EDITLINE_COMPENTRY_FUNC_RETURNS_CHARP */
     ::using_history();
 
-    std::string lang = solver->getOption("input-language");
+    std::string lang = d_solver->getOption("input-language");
     if (lang == "LANG_TPTP")
     {
       d_historyFilename = string(getenv("HOME")) + "/.cvc5_history_tptp";
@@ -173,11 +178,10 @@ InteractiveShell::~InteractiveShell() {
 #endif /* HAVE_LIBEDITLINE */
 }
 
-std::optional<InteractiveShell::CmdSeq> InteractiveShell::readCommand()
+bool InteractiveShell::readAndExecCommands()
 {
   char* lineBuf = NULL;
   string line = "";
-
 restart:
 
   /* Don't do anything if the input is closed or if we've seen a
@@ -187,7 +191,7 @@ restart:
     {
       d_out << endl;
     }
-    return {};
+    return false;
   }
 
   /* If something's wrong with the input, there's nothing we can do. */
@@ -262,7 +266,7 @@ restart:
         {
           d_out << endl;
         }
-        return {};
+        return false;
       }
 
       /* Some input left to parse, but nothing left to read.
@@ -318,19 +322,29 @@ restart:
     }
   }
 
-  d_parser->setStringInput(
-      d_solver->getOption("input-language"), input, INPUT_FILENAME);
+  d_parser->appendIncrementalStringInput(input);
 
   /* There may be more than one command in the input. Build up a
      sequence. */
   std::vector<std::unique_ptr<Command>> cmdSeq;
-  Command *cmd;
+  std::unique_ptr<Command> cmdp;
+  // remember the scope level of the symbol manager, in case we hit an end of
+  // line (when catching ParserEndOfFileException).
+  size_t lastScopeLevel = d_symman->scopeLevel();
 
   try
   {
-    while ((cmd = d_parser->nextCommand()))
+    while ((cmdp = d_parser->nextCommand()))
     {
-      cmdSeq.emplace_back(cmd);
+      Command* cmd = cmdp.get();
+      // execute the command immediately
+      d_cexec->doCommand(cmd);
+      if (cmd->interrupted())
+      {
+        d_quit = true;
+        return false;
+      }
+      cmdSeq.emplace_back(std::move(cmdp));
       if (dynamic_cast<QuitCommand*>(cmd) != NULL)
       {
         d_quit = true;
@@ -361,10 +375,16 @@ restart:
         }
 #endif /* HAVE_LIBEDITLINE */
       }
+      lastScopeLevel = d_symman->scopeLevel();
     }
   }
   catch (ParserEndOfFileException& pe)
   {
+    // pop back to the scope we were at prior to reading the last command
+    while (d_symman->scopeLevel() > lastScopeLevel)
+    {
+      d_symman->popScope();
+    }
     line += "\n";
     goto restart;
   }
@@ -382,7 +402,7 @@ restart:
     if (!d_isInteractive)
     {
       d_quit = true;
-      return {};
+      return false;
     }
     // We can't really clear out the sequence and abort the current line,
     // because the parse error might be for the second command on the
@@ -399,7 +419,7 @@ restart:
     // cmd_seq = new CommandSequence();
   }
 
-  return std::optional<CmdSeq>(std::move(cmdSeq));
+  return true;
 }/* InteractiveShell::readCommand() */
 
 #if HAVE_LIBEDITLINE
