@@ -15,16 +15,16 @@
 
 #include "cvc5_public.h"
 
-#ifndef CVC5__EXPR__SYMBOL_MANAGER_H
-#define CVC5__EXPR__SYMBOL_MANAGER_H
+#ifndef CVC5__PARSER__API__CPP__SYMBOL_MANAGER_H
+#define CVC5__PARSER__API__CPP__SYMBOL_MANAGER_H
+
+#include <cvc5/cvc5.h>
+#include <cvc5/cvc5_export.h>
 
 #include <map>
 #include <memory>
 #include <string>
 
-#include "api/cpp/cvc5.h"
-#include "cvc5_export.h"
-
 namespace cvc5 {
 
 namespace internal::parser {
@@ -83,6 +83,33 @@ class CVC5_EXPORT SymbolManager
    */
   bool bind(const std::string& name, cvc5::Term obj, bool doOverload = false);
 
+  /**
+   * Bind a type to a name in the current scope.  If <code>name</code>
+   * is already bound to a type in the current level, then the binding
+   * is replaced. If <code>name</code> is bound in a previous level,
+   * then the binding is "covered" by this one until the current scope
+   * is popped.
+   *
+   * @param name an identifier
+   * @param t the type to bind to <code>name</code>
+   */
+  void bindType(const std::string& name, cvc5::Sort t);
+
+  /**
+   * Bind a type to a name in the current scope.  If <code>name</code>
+   * is already bound to a type or type constructor in the current
+   * level, then the binding is replaced. If <code>name</code> is
+   * bound in a previous level, then the binding is "covered" by this
+   * one until the current scope is popped.
+   *
+   * @param name an identifier
+   * @param params the parameters to the type
+   * @param t the type to bind to <code>name</code>
+   */
+  void bindType(const std::string& name,
+                const std::vector<cvc5::Sort>& params,
+                cvc5::Sort t);
+
   //---------------------------- named expressions
   /** Set name of term t to name
    *
