@@ -16,12 +16,13 @@
 
 #include "parser/symbol_table.h"
 
+#include <cvc5/cvc5.h>
+
 #include <ostream>
 #include <string>
 #include <unordered_map>
 #include <utility>
 
-#include "api/cpp/cvc5.h"
 #include "base/check.h"
 #include "context/cdhashmap.h"
 #include "context/cdhashset.h"
@@ -92,34 +93,32 @@ class OverloadedTypeTrie
 {
  public:
   OverloadedTypeTrie(Context* c, bool allowFunVariants = false)
-      : d_overloaded_symbols(
-          new (true) CDHashSet<cvc5::Term, std::hash<cvc5::Term>>(c)),
+      : d_overloaded_symbols(new (true) CDHashSet<Term, std::hash<Term>>(c)),
         d_allowFunctionVariants(allowFunVariants)
   {
   }
   ~OverloadedTypeTrie() { d_overloaded_symbols->deleteSelf(); }
 
   /** is this function overloaded? */
-  bool isOverloadedFunction(cvc5::Term fun) const;
+  bool isOverloadedFunction(Term fun) const;
 
   /** Get overloaded constant for type.
    * If possible, it returns a defined symbol with name
    * that has type t. Otherwise returns null expression.
    */
-  cvc5::Term getOverloadedConstantForType(const std::string& name,
-                                          cvc5::Sort t) const;
+  Term getOverloadedConstantForType(const std::string& name, Sort t) const;
 
   /**
    * If possible, returns a defined function for a name
    * and a vector of expected argument types. Otherwise returns
    * null expression.
    */
-  cvc5::Term getOverloadedFunctionForTypes(
-      const std::string& name, const std::vector<cvc5::Sort>& argTypes) const;
+  Term getOverloadedFunctionForTypes(const std::string& name,
+                                     const std::vector<Sort>& argTypes) const;
   /** called when obj is bound to name, and prev_bound_obj was already bound to
    * name Returns false if the binding is invalid.
    */
-  bool bind(const string& name, cvc5::Term prev_bound_obj, cvc5::Term obj);
+  bool bind(const string& name, Term prev_bound_obj, Term obj);
 
  private:
   /** Marks expression obj with name as overloaded.
@@ -137,9 +136,9 @@ class OverloadedTypeTrie
    * These are put in the same place in the trie but do not have identical type,
    * hence we return false.
    */
-  bool markOverloaded(const string& name, cvc5::Term obj);
+  bool markOverloaded(const string& name, Term obj);
   /** the null expression */
-  cvc5::Term d_nullTerm;
+  Term d_nullTerm;
   // The (context-independent) trie storing that maps expected argument
   // vectors to symbols. All expressions stored in d_symbols are only
   // interpreted as active if they also appear in the context-dependent
@@ -148,15 +147,15 @@ class OverloadedTypeTrie
   {
    public:
     // children of this node
-    std::map<cvc5::Sort, TypeArgTrie> d_children;
+    std::map<Sort, TypeArgTrie> d_children;
     // symbols at this node
-    std::map<cvc5::Sort, cvc5::Term> d_symbols;
+    std::map<Sort, Term> d_symbols;
   };
   /** for each string with operator overloading, this stores the data structure
    * above. */
   std::unordered_map<std::string, TypeArgTrie> d_overload_type_arg_trie;
   /** The set of overloaded symbols. */
-  CDHashSet<cvc5::Term, std::hash<cvc5::Term>>* d_overloaded_symbols;
+  CDHashSet<Term, std::hash<Term>>* d_overloaded_symbols;
   /** allow function variants
    * This is true if we allow overloading (non-constant) functions that expect
    * the same argument types.
@@ -168,27 +167,26 @@ class OverloadedTypeTrie
    * if reqUnique=true.
    * Otherwise, it returns the null expression.
    */
-  cvc5::Term getOverloadedFunctionAt(const TypeArgTrie* tat,
-                                     bool reqUnique = true) const;
+  Term getOverloadedFunctionAt(const TypeArgTrie* tat,
+                               bool reqUnique = true) const;
 };
 
-bool OverloadedTypeTrie::isOverloadedFunction(cvc5::Term fun) const
+bool OverloadedTypeTrie::isOverloadedFunction(Term fun) const
 {
   return d_overloaded_symbols->find(fun) != d_overloaded_symbols->end();
 }
 
-cvc5::Term OverloadedTypeTrie::getOverloadedConstantForType(
-    const std::string& name, cvc5::Sort t) const
+Term OverloadedTypeTrie::getOverloadedConstantForType(const std::string& name,
+                                                      Sort t) const
 {
   std::unordered_map<std::string, TypeArgTrie>::const_iterator it =
       d_overload_type_arg_trie.find(name);
   if (it != d_overload_type_arg_trie.end())
   {
-    std::map<cvc5::Sort, cvc5::Term>::const_iterator its =
-        it->second.d_symbols.find(t);
+    std::map<Sort, Term>::const_iterator its = it->second.d_symbols.find(t);
     if (its != it->second.d_symbols.end())
     {
-      cvc5::Term expr = its->second;
+      Term expr = its->second;
       // must be an active symbol
       if (isOverloadedFunction(expr))
       {
@@ -199,8 +197,8 @@ cvc5::Term OverloadedTypeTrie::getOverloadedConstantForType(
   return d_nullTerm;
 }
 
-cvc5::Term OverloadedTypeTrie::getOverloadedFunctionForTypes(
-    const std::string& name, const std::vector<cvc5::Sort>& argTypes) const
+Term OverloadedTypeTrie::getOverloadedFunctionForTypes(
+    const std::string& name, const std::vector<Sort>& argTypes) const
 {
   std::unordered_map<std::string, TypeArgTrie>::const_iterator it =
       d_overload_type_arg_trie.find(name);
@@ -209,7 +207,7 @@ cvc5::Term OverloadedTypeTrie::getOverloadedFunctionForTypes(
     const TypeArgTrie* tat = &it->second;
     for (unsigned i = 0; i < argTypes.size(); i++)
     {
-      std::map<cvc5::Sort, TypeArgTrie>::const_iterator itc =
+      std::map<Sort, TypeArgTrie>::const_iterator itc =
           tat->d_children.find(argTypes[i]);
       if (itc != tat->d_children.end())
       {
@@ -230,9 +228,7 @@ cvc5::Term OverloadedTypeTrie::getOverloadedFunctionForTypes(
   return d_nullTerm;
 }
 
-bool OverloadedTypeTrie::bind(const string& name,
-                              cvc5::Term prev_bound_obj,
-                              cvc5::Term obj)
+bool OverloadedTypeTrie::bind(const string& name, Term prev_bound_obj, Term obj)
 {
   Assert(prev_bound_obj != obj);
   bool retprev = true;
@@ -246,15 +242,15 @@ bool OverloadedTypeTrie::bind(const string& name,
   return retprev && retobj;
 }
 
-bool OverloadedTypeTrie::markOverloaded(const string& name, cvc5::Term obj)
+bool OverloadedTypeTrie::markOverloaded(const string& name, Term obj)
 {
   Trace("parser-overloading") << "Overloaded function : " << name;
   Trace("parser-overloading")
       << " with type " << obj.getSort() << ", obj is " << obj << std::endl;
   // get the argument types
-  cvc5::Sort t = obj.getSort();
-  cvc5::Sort rangeType = t;
-  std::vector<cvc5::Sort> argTypes;
+  Sort t = obj.getSort();
+  Sort rangeType = t;
+  std::vector<Sort> argTypes;
   if (t.isFunction())
   {
     argTypes = t.getFunctionDomainSorts();
@@ -281,16 +277,16 @@ bool OverloadedTypeTrie::markOverloaded(const string& name, cvc5::Term obj)
   if (d_allowFunctionVariants || argTypes.empty())
   {
     // they are allowed, check for redefinition
-    std::map<cvc5::Sort, cvc5::Term>::iterator it =
-        tat->d_symbols.find(rangeType);
+    std::map<Sort, Term>::iterator it = tat->d_symbols.find(rangeType);
     if (it != tat->d_symbols.end())
     {
-      cvc5::Term prev_obj = it->second;
+      Term prev_obj = it->second;
       // if there is already an active function with the same name and expects
       // the same argument types and has the same return type, we reject the
       // re-declaration here.
       if (isOverloadedFunction(prev_obj))
       {
+        Trace("parser-overloading") << "...existing overload" << std::endl;
         return false;
       }
     }
@@ -298,12 +294,15 @@ bool OverloadedTypeTrie::markOverloaded(const string& name, cvc5::Term obj)
   else
   {
     // they are not allowed, we cannot have any function defined here.
-    cvc5::Term existingFun = getOverloadedFunctionAt(tat, false);
+    Term existingFun = getOverloadedFunctionAt(tat, false);
     if (!existingFun.isNull())
     {
+      Trace("parser-overloading")
+          << "...existing fun (no variant)" << std::endl;
       return false;
     }
   }
+  Trace("parser-overloading") << "...update symbols" << std::endl;
 
   // otherwise, update the symbols
   d_overloaded_symbols->insert(obj);
@@ -311,16 +310,15 @@ bool OverloadedTypeTrie::markOverloaded(const string& name, cvc5::Term obj)
   return true;
 }
 
-cvc5::Term OverloadedTypeTrie::getOverloadedFunctionAt(
+Term OverloadedTypeTrie::getOverloadedFunctionAt(
     const OverloadedTypeTrie::TypeArgTrie* tat, bool reqUnique) const
 {
-  cvc5::Term retExpr;
-  for (std::map<cvc5::Sort, cvc5::Term>::const_iterator its =
-           tat->d_symbols.begin();
+  Term retExpr;
+  for (std::map<Sort, Term>::const_iterator its = tat->d_symbols.begin();
        its != tat->d_symbols.end();
        ++its)
   {
-    cvc5::Term expr = its->second;
+    Term expr = its->second;
     if (isOverloadedFunction(expr))
     {
       if (retExpr.isNull())
@@ -357,17 +355,14 @@ class SymbolTable::Implementation
 
   ~Implementation() {}
 
-  bool bind(const string& name, cvc5::Term obj, bool doOverload);
-  void bindType(const string& name, cvc5::Sort t);
-  void bindType(const string& name,
-                const vector<cvc5::Sort>& params,
-                cvc5::Sort t);
+  bool bind(const string& name, Term obj, bool doOverload);
+  void bindType(const string& name, Sort t);
+  void bindType(const string& name, const vector<Sort>& params, Sort t);
   bool isBound(const string& name) const;
   bool isBoundType(const string& name) const;
-  cvc5::Term lookup(const string& name) const;
-  cvc5::Sort lookupType(const string& name) const;
-  cvc5::Sort lookupType(const string& name,
-                        const vector<cvc5::Sort>& params) const;
+  Term lookup(const string& name) const;
+  Sort lookupType(const string& name) const;
+  Sort lookupType(const string& name, const vector<Sort>& params) const;
   size_t lookupArity(const string& name);
   void popScope();
   void pushScope();
@@ -376,30 +371,29 @@ class SymbolTable::Implementation
   void resetAssertions();
   //------------------------ operator overloading
   /** implementation of function from header */
-  bool isOverloadedFunction(cvc5::Term fun) const;
+  bool isOverloadedFunction(Term fun) const;
 
   /** implementation of function from header */
-  cvc5::Term getOverloadedConstantForType(const std::string& name,
-                                          cvc5::Sort t) const;
+  Term getOverloadedConstantForType(const std::string& name, Sort t) const;
 
   /** implementation of function from header */
-  cvc5::Term getOverloadedFunctionForTypes(
-      const std::string& name, const std::vector<cvc5::Sort>& argTypes) const;
+  Term getOverloadedFunctionForTypes(const std::string& name,
+                                     const std::vector<Sort>& argTypes) const;
   //------------------------ end operator overloading
  private:
   /** The context manager for the scope maps. */
   Context d_context;
 
   /** A map for expressions. */
-  CDHashMap<string, cvc5::Term> d_exprMap;
+  CDHashMap<string, Term> d_exprMap;
 
   /** A map for types. */
-  using TypeMap = CDHashMap<string, std::pair<vector<cvc5::Sort>, cvc5::Sort>>;
+  using TypeMap = CDHashMap<string, std::pair<vector<Sort>, Sort>>;
   TypeMap d_typeMap;
 
   //------------------------ operator overloading
   // the null expression
-  cvc5::Term d_nullTerm;
+  Term d_nullTerm;
   // overloaded type trie, stores all information regarding overloading
   OverloadedTypeTrie d_overload_trie;
   /** bind with overloading
@@ -407,15 +401,15 @@ class SymbolTable::Implementation
    * allowed. If a symbol is previously bound to that name, it marks both as
    * overloaded. Returns false if the binding was invalid.
    */
-  bool bindWithOverloading(const string& name, cvc5::Term obj);
+  bool bindWithOverloading(const string& name, Term obj);
   //------------------------ end operator overloading
 }; /* SymbolTable::Implementation */
 
 bool SymbolTable::Implementation::bind(const string& name,
-                                       cvc5::Term obj,
+                                       Term obj,
                                        bool doOverload)
 {
-  Assert(!obj.isNull()) << "cannot bind to a null cvc5::Term";
+  Assert(!obj.isNull()) << "cannot bind to a null Term";
   Trace("sym-table") << "SymbolTable: bind " << name << " to " << obj
                      << ", doOverload=" << doOverload << std::endl;
   if (doOverload)
@@ -435,10 +429,10 @@ bool SymbolTable::Implementation::isBound(const string& name) const
   return d_exprMap.find(name) != d_exprMap.end();
 }
 
-cvc5::Term SymbolTable::Implementation::lookup(const string& name) const
+Term SymbolTable::Implementation::lookup(const string& name) const
 {
   Assert(isBound(name));
-  cvc5::Term expr = (*d_exprMap.find(name)).second;
+  Term expr = (*d_exprMap.find(name)).second;
   if (isOverloadedFunction(expr))
   {
     return d_nullTerm;
@@ -449,14 +443,14 @@ cvc5::Term SymbolTable::Implementation::lookup(const string& name) const
   }
 }
 
-void SymbolTable::Implementation::bindType(const string& name, cvc5::Sort t)
+void SymbolTable::Implementation::bindType(const string& name, Sort t)
 {
-  d_typeMap.insert(name, make_pair(vector<cvc5::Sort>(), t));
+  d_typeMap.insert(name, make_pair(vector<Sort>(), t));
 }
 
 void SymbolTable::Implementation::bindType(const string& name,
-                                           const vector<cvc5::Sort>& params,
-                                           cvc5::Sort t)
+                                           const vector<Sort>& params,
+                                           Sort t)
 {
   if (TraceIsOn("sort"))
   {
@@ -465,7 +459,7 @@ void SymbolTable::Implementation::bindType(const string& name,
     {
       copy(params.begin(),
            params.end() - 1,
-           ostream_iterator<cvc5::Sort>(Trace("sort"), ", "));
+           ostream_iterator<Sort>(Trace("sort"), ", "));
       Trace("sort") << params.back();
     }
     Trace("sort") << "], " << t << ")" << endl;
@@ -479,21 +473,19 @@ bool SymbolTable::Implementation::isBoundType(const string& name) const
   return d_typeMap.find(name) != d_typeMap.end();
 }
 
-cvc5::Sort SymbolTable::Implementation::lookupType(const string& name) const
+Sort SymbolTable::Implementation::lookupType(const string& name) const
 {
-  std::pair<std::vector<cvc5::Sort>, cvc5::Sort> p =
-      (*d_typeMap.find(name)).second;
+  std::pair<std::vector<Sort>, Sort> p = (*d_typeMap.find(name)).second;
   Assert(p.first.size() == 0)
       << "type constructor arity is wrong: `" << name << "' requires "
       << p.first.size() << " parameters but was provided 0";
   return p.second;
 }
 
-cvc5::Sort SymbolTable::Implementation::lookupType(
-    const string& name, const vector<cvc5::Sort>& params) const
+Sort SymbolTable::Implementation::lookupType(const string& name,
+                                             const vector<Sort>& params) const
 {
-  std::pair<std::vector<cvc5::Sort>, cvc5::Sort> p =
-      (*d_typeMap.find(name)).second;
+  std::pair<std::vector<Sort>, Sort> p = (*d_typeMap.find(name)).second;
   Assert(p.first.size() == params.size())
       << "type constructor arity is wrong: `" << name.c_str() << "' requires "
       << p.first.size() << " parameters but was provided " << params.size();
@@ -519,18 +511,18 @@ cvc5::Sort SymbolTable::Implementation::lookupType(
     Trace("sort") << "have formals [";
     copy(p.first.begin(),
          p.first.end() - 1,
-         ostream_iterator<cvc5::Sort>(Trace("sort"), ", "));
+         ostream_iterator<Sort>(Trace("sort"), ", "));
     Trace("sort") << p.first.back() << "]" << std::endl << "parameters   [";
     copy(params.begin(),
          params.end() - 1,
-         ostream_iterator<cvc5::Sort>(Trace("sort"), ", "));
+         ostream_iterator<Sort>(Trace("sort"), ", "));
     Trace("sort") << params.back() << "]" << endl
                   << "type ctor    " << name << std::endl
                   << "type is      " << p.second << std::endl;
   }
-  cvc5::Sort instantiation = isUninterpretedSortConstructor
-                                 ? p.second.instantiate(params)
-                                 : p.second.substitute(p.first, params);
+  Sort instantiation = isUninterpretedSortConstructor
+                           ? p.second.instantiate(params)
+                           : p.second.substitute(p.first, params);
   Trace("sort") << "instance is  " << instantiation << std::endl;
 
   return instantiation;
@@ -538,8 +530,7 @@ cvc5::Sort SymbolTable::Implementation::lookupType(
 
 size_t SymbolTable::Implementation::lookupArity(const string& name)
 {
-  std::pair<std::vector<cvc5::Sort>, cvc5::Sort> p =
-      (*d_typeMap.find(name)).second;
+  std::pair<std::vector<Sort>, Sort> p = (*d_typeMap.find(name)).second;
   return p.first.size();
 }
 
@@ -578,30 +569,30 @@ void SymbolTable::Implementation::resetAssertions()
   d_context.push();
 }
 
-bool SymbolTable::Implementation::isOverloadedFunction(cvc5::Term fun) const
+bool SymbolTable::Implementation::isOverloadedFunction(Term fun) const
 {
   return d_overload_trie.isOverloadedFunction(fun);
 }
 
-cvc5::Term SymbolTable::Implementation::getOverloadedConstantForType(
-    const std::string& name, cvc5::Sort t) const
+Term SymbolTable::Implementation::getOverloadedConstantForType(
+    const std::string& name, Sort t) const
 {
   return d_overload_trie.getOverloadedConstantForType(name, t);
 }
 
-cvc5::Term SymbolTable::Implementation::getOverloadedFunctionForTypes(
-    const std::string& name, const std::vector<cvc5::Sort>& argTypes) const
+Term SymbolTable::Implementation::getOverloadedFunctionForTypes(
+    const std::string& name, const std::vector<Sort>& argTypes) const
 {
   return d_overload_trie.getOverloadedFunctionForTypes(name, argTypes);
 }
 
 bool SymbolTable::Implementation::bindWithOverloading(const string& name,
-                                                      cvc5::Term obj)
+                                                      Term obj)
 {
-  CDHashMap<string, cvc5::Term>::const_iterator it = d_exprMap.find(name);
+  CDHashMap<string, Term>::const_iterator it = d_exprMap.find(name);
   if (it != d_exprMap.end())
   {
-    const cvc5::Term& prev_bound_obj = (*it).second;
+    const Term& prev_bound_obj = (*it).second;
     if (prev_bound_obj != obj)
     {
       return d_overload_trie.bind(name, prev_bound_obj, obj);
@@ -610,19 +601,19 @@ bool SymbolTable::Implementation::bindWithOverloading(const string& name,
   return true;
 }
 
-bool SymbolTable::isOverloadedFunction(cvc5::Term fun) const
+bool SymbolTable::isOverloadedFunction(Term fun) const
 {
   return d_implementation->isOverloadedFunction(fun);
 }
 
-cvc5::Term SymbolTable::getOverloadedConstantForType(const std::string& name,
-                                                     cvc5::Sort t) const
+Term SymbolTable::getOverloadedConstantForType(const std::string& name,
+                                               Sort t) const
 {
   return d_implementation->getOverloadedConstantForType(name, t);
 }
 
-cvc5::Term SymbolTable::getOverloadedFunctionForTypes(
-    const std::string& name, const std::vector<cvc5::Sort>& argTypes) const
+Term SymbolTable::getOverloadedFunctionForTypes(
+    const std::string& name, const std::vector<Sort>& argTypes) const
 {
   return d_implementation->getOverloadedFunctionForTypes(name, argTypes);
 }
@@ -632,19 +623,19 @@ SymbolTable::SymbolTable() : d_implementation(new SymbolTable::Implementation())
 }
 
 SymbolTable::~SymbolTable() {}
-bool SymbolTable::bind(const string& name, cvc5::Term obj, bool doOverload)
+bool SymbolTable::bind(const string& name, Term obj, bool doOverload)
 {
   return d_implementation->bind(name, obj, doOverload);
 }
 
-void SymbolTable::bindType(const string& name, cvc5::Sort t)
+void SymbolTable::bindType(const string& name, Sort t)
 {
   d_implementation->bindType(name, t);
 }
 
 void SymbolTable::bindType(const string& name,
-                           const vector<cvc5::Sort>& params,
-                           cvc5::Sort t)
+                           const vector<Sort>& params,
+                           Sort t)
 {
   d_implementation->bindType(name, params, t);
 }
@@ -657,17 +648,17 @@ bool SymbolTable::isBoundType(const string& name) const
 {
   return d_implementation->isBoundType(name);
 }
-cvc5::Term SymbolTable::lookup(const string& name) const
+Term SymbolTable::lookup(const string& name) const
 {
   return d_implementation->lookup(name);
 }
-cvc5::Sort SymbolTable::lookupType(const string& name) const
+Sort SymbolTable::lookupType(const string& name) const
 {
   return d_implementation->lookupType(name);
 }
 
-cvc5::Sort SymbolTable::lookupType(const string& name,
-                                   const vector<cvc5::Sort>& params) const
+Sort SymbolTable::lookupType(const string& name,
+                             const vector<Sort>& params) const
 {
   return d_implementation->lookupType(name, params);
 }
