@@ -396,6 +396,44 @@ bool LfscProofPostprocessCallback::update(Node res,
       addLfscRule(cdp, falsen, children, LfscRule::CONCAT_CONFLICT_DEQ, args);
     }
     break;
+    case PfRule::INSTANTIATE:
+    {
+      Node q = children[0];
+      Assert(q.getKind() == FORALL);
+      std::vector<Node> terms;
+      std::vector<Node> qvars(q[0].begin(), q[0].end());
+      Node conc = q;
+      for (size_t i = 0, nvars = q[0].getNumChildren(); i < nvars; i++)
+      {
+        Assert(conc.getKind() == FORALL);
+        Node prevConc = conc;
+        if (i + 1 == nvars)
+        {
+          conc = res;
+        }
+        else
+        {
+          Assert(i + 1 < qvars.size());
+          std::vector<Node> qvarsNew(qvars.begin() + i + 1, qvars.end());
+          Assert(!qvarsNew.empty());
+          std::vector<Node> qchildren;
+          TNode v = qvars[i];
+          TNode subs = args[i];
+          qchildren.push_back(nm->mkNode(BOUND_VAR_LIST, qvarsNew));
+          qchildren.push_back(conc[1].substitute(v, subs));
+          conc = nm->mkNode(FORALL, qchildren);
+        }
+        addLfscRule(cdp, conc, {prevConc}, LfscRule::INSTANTIATE, {args[i]});
+      }
+    }
+    break;
+    case PfRule::BETA_REDUCE:
+    {
+      // get the term to beta-reduce
+      Node termToReduce = nm->mkNode(APPLY_UF, args);
+      addLfscRule(cdp, res, {}, LfscRule::BETA_REDUCE, {termToReduce});
+    }
+    break;
     default: return false; break;
   }
   AlwaysAssert(cdp->getProofFor(res)->getRule() != PfRule::ASSUME);
