@@ -146,7 +146,11 @@ void CnfStream::ensureLiteral(TNode n)
   }
 }
 
-SatLiteral CnfStream::newLiteral(TNode node, bool isTheoryAtom, bool preRegister, bool canEliminate) {
+SatLiteral CnfStream::newLiteral(TNode node,
+                                 bool isTheoryAtom,
+                                 bool notifyTheory,
+                                 bool canEliminate)
+{
   Trace("cnf") << d_name << "::newLiteral(" << node << ", " << isTheoryAtom
                << ")\n"
                << push;
@@ -161,38 +165,50 @@ SatLiteral CnfStream::newLiteral(TNode node, bool isTheoryAtom, bool preRegister
 
   // Get the literal for this node
   SatLiteral lit;
-  if (!hasLiteral(node)) {
+  if (!hasLiteral(node))
+  {
     Trace("cnf") << d_name << "::newLiteral: node already registered\n";
     // If no literal, we'll make one
-    if (node.getKind() == kind::CONST_BOOLEAN) {
+    if (node.getKind() == kind::CONST_BOOLEAN)
+    {
       Trace("cnf") << d_name << "::newLiteral: boolean const\n";
-      if (node.getConst<bool>()) {
+      if (node.getConst<bool>())
+      {
         lit = SatLiteral(d_satSolver->trueVar());
-      } else {
+      }
+      else
+      {
         lit = SatLiteral(d_satSolver->falseVar());
       }
-    } else {
+    }
+    else
+    {
       Trace("cnf") << d_name << "::newLiteral: new var\n";
-      lit = SatLiteral(d_satSolver->newVar(isTheoryAtom, preRegister, canEliminate));
+      lit = SatLiteral(d_satSolver->newVar(isTheoryAtom, canEliminate));
     }
     d_nodeToLiteralMap.insert(node, lit);
     d_nodeToLiteralMap.insert(node.notNode(), ~lit);
-  } else {
+  }
+  else
+  {
+    Trace("cnf") << d_name << "::newLiteral: node already registered\n";
     lit = getLiteral(node);
   }
 
   // If it's a theory literal, need to store it for back queries
-  if (isTheoryAtom || d_flitPolicy == FormulaLitPolicy::TRACK)
+  if (isTheoryAtom || d_flitPolicy == FormulaLitPolicy::TRACK
+      || d_flitPolicy == FormulaLitPolicy::TRACK_AND_NOTIFY_VAR)
   {
     d_literalToNodeMap.insert_safe(lit, node);
     d_literalToNodeMap.insert_safe(~lit, node.notNode());
   }
 
   // If a theory literal, we pre-register it
-  if (preRegister) {
+  if (notifyTheory)
+  {
     // In case we are re-entered due to lemmas, save our state
     bool backupRemovable = d_removable;
-    d_registrar->preRegister(node);
+    d_registrar->notifySatLiteral(node);
     d_removable = backupRemovable;
   }
   // Here, you can have it
@@ -200,7 +216,9 @@ SatLiteral CnfStream::newLiteral(TNode node, bool isTheoryAtom, bool preRegister
   return lit;
 }
 
-TNode CnfStream::getNode(const SatLiteral& literal) {
+TNode CnfStream::getNode(const SatLiteral& literal)
+{
+  Assert(d_literalToNodeMap.find(literal) != d_literalToNodeMap.end());
   Trace("cnf") << "getNode(" << literal << ")\n";
   Trace("cnf") << "getNode(" << literal << ") => "
                << d_literalToNodeMap[literal] << "\n";
@@ -242,6 +260,12 @@ SatLiteral CnfStream::convertAtom(TNode node)
   if (node.isVar() && node.getKind() != kind::BOOLEAN_TERM_VARIABLE)
   {
     d_booleanVariables.push_back(node);
+    // if TRACK_AND_NOTIFY_VAR, we are notified when Boolean variables are
+    // asserted. Thus, they are marked as theory literals.
+    if (d_flitPolicy == FormulaLitPolicy::TRACK_AND_NOTIFY_VAR)
+    {
+      theoryLiteral = true;
+    }
   }
   else
   {
