@@ -50,6 +50,8 @@ enum class FormulaLitPolicy : uint32_t
 {
   // literals for formulas are notified
   TRACK_AND_NOTIFY,
+  // literals for Boolean variables are notified
+  TRACK_AND_NOTIFY_VAR,
   // literals for formulas are added to node map
   TRACK,
   // literals for formulas are kept internal (default)
@@ -203,42 +205,6 @@ class CnfStream : protected EnvObj
    */
   void ensureMappingForLiteral(TNode n);
 
-  /** The SAT solver we will be using */
-  SatSolver* d_satSolver;
-
-  /** Boolean variables that we translated */
-  context::CDList<TNode> d_booleanVariables;
-
-  /** Formulas that we translated that we are notifying */
-  context::CDHashSet<Node> d_notifyFormulas;
-
-  /** Map from nodes to literals */
-  NodeToLiteralMap d_nodeToLiteralMap;
-
-  /** Map from literals to nodes */
-  LiteralToNodeMap d_literalToNodeMap;
-
-  /**
-   * True if the lit-to-Node map should be kept for all lits, not just
-   * theory lits.  This is true if e.g. replay logging is on, which
-   * dumps the Nodes corresponding to decision literals.
-   */
-  const FormulaLitPolicy d_flitPolicy;
-
-  /** The "registrar" for pre-registration of terms */
-  Registrar* d_registrar;
-
-  /** The name of this CNF stream*/
-  std::string d_name;
-
-  /**
-   * Are we asserting a removable clause (true) or a permanent clause (false).
-   * This is set at the beginning of convertAndAssert so that it doesn't
-   * need to be passed on over the stack.  Only pure clauses can be asserted
-   * as removable.
-   */
-  bool d_removable;
-
   /**
    * Asserts the given clause to the sat solver.
    * @param node the node giving rise to this clause
@@ -280,13 +246,15 @@ class CnfStream : protected EnvObj
    * @param node a formula
    * @param isTheoryAtom is this a theory atom that needs to be asserted to
    * theory.
-   * @param preRegister whether to preregister the atom with the theory
+   * @param notifyTheory whether to notify the theory of the atom
    * @param canEliminate whether the sat solver can safely eliminate this
    * variable.
    * @return the literal corresponding to the formula
    */
-  SatLiteral newLiteral(TNode node, bool isTheoryAtom = false,
-                        bool preRegister = false, bool canEliminate = true);
+  SatLiteral newLiteral(TNode node,
+                        bool isTheoryAtom = false,
+                        bool notifyTheory = false,
+                        bool canEliminate = true);
 
   /**
    * Constructs a new literal for an atom and returns it.  Calls
@@ -298,6 +266,42 @@ class CnfStream : protected EnvObj
    */
   SatLiteral convertAtom(TNode node);
 
+  /** The SAT solver we will be using */
+  SatSolver* d_satSolver;
+
+  /** Boolean variables that we translated */
+  context::CDList<TNode> d_booleanVariables;
+
+  /** Formulas that we translated that we are notifying */
+  context::CDHashSet<Node> d_notifyFormulas;
+
+  /** Map from nodes to literals */
+  NodeToLiteralMap d_nodeToLiteralMap;
+
+  /** Map from literals to nodes */
+  LiteralToNodeMap d_literalToNodeMap;
+
+  /**
+   * True if the lit-to-Node map should be kept for all lits, not just
+   * theory lits.  This is true if e.g. replay logging is on, which
+   * dumps the Nodes corresponding to decision literals.
+   */
+  const FormulaLitPolicy d_flitPolicy;
+
+  /** The "registrar" for pre-registration of terms */
+  Registrar* d_registrar;
+
+  /** The name of this CNF stream*/
+  std::string d_name;
+
+  /**
+   * Are we asserting a removable clause (true) or a permanent clause (false).
+   * This is set at the beginning of convertAndAssert so that it doesn't
+   * need to be passed on over the stack.  Only pure clauses can be asserted
+   * as removable.
+   */
+  bool d_removable;
+
   /** Pointer to resource manager for associated SolverEngine */
   ResourceManager* d_resourceManager;
 
