@@ -63,6 +63,27 @@ bool ProofCnfStream::isBlocked(std::shared_ptr<ProofNode> pfn)
   return d_blocked.contains(pfn);
 }
 
+std::vector<Node> ProofCnfStream::getInputClauses()
+{
+  std::vector<Node> cls;
+  for (const Node& c : d_inputClauses)
+  {
+    cls.push_back(c);
+  }
+  return cls;
+}
+
+std::vector<Node> ProofCnfStream::getLemmaClauses()
+{
+  std::vector<Node> cls;
+  for (const Node& c : d_lemmaClauses)
+  {
+    cls.push_back(c);
+  }
+
+  return cls;
+}
+
 std::vector<std::shared_ptr<ProofNode>> ProofCnfStream::getInputClausesProofs()
 {
   std::vector<std::shared_ptr<ProofNode>> pfs;
@@ -675,7 +696,7 @@ void ProofCnfStream::convertPropagation(TrustNode trn)
   }
 }
 
-void ProofCnfStream::notifyCurrPropagationInsertedAtLevel(int explLevel)
+void ProofCnfStream::notifyCurrPropagationInsertedAtLevel(uint32_t explLevel)
 {
   Assert(explLevel < (userContext()->getLevel() - 1));
   Assert(!d_currPropagationProcessed.isNull());
@@ -709,7 +730,7 @@ void ProofCnfStream::notifyCurrPropagationInsertedAtLevel(int explLevel)
 }
 
 void ProofCnfStream::notifyClauseInsertedAtLevel(const SatClause& clause,
-                                                 int clLevel)
+                                                 uint32_t clLevel)
 {
   Trace("cnf") << "Need to save clause " << clause << " in level "
                << clLevel + 1 << " despite being currently in level "
