@@ -53,7 +53,9 @@ TheoryProxy::TheoryProxy(Env& env,
       d_tpp(env, *theoryEngine),
       d_skdm(skdm),
       d_zll(nullptr),
-      d_stopSearch(false, userContext())
+      d_prr(nullptr),
+      d_stopSearch(false, userContext()),
+      d_activatedSkDefs(false)
 {
   bool trackZeroLevel =
       options().smt.deepRestartMode != options::DeepRestartMode::NONE
@@ -70,7 +72,7 @@ TheoryProxy::~TheoryProxy() {
   /* nothing to do for now */
 }
 
-void TheoryProxy::finishInit(CDCLTSatSolverInterface* ss, CnfStream* cs)
+void TheoryProxy::finishInit(CDCLTSatSolver* ss, CnfStream* cs)
 {
   // make the decision engine, which requires pointers to the SAT solver and CNF
   // stream
@@ -84,8 +86,11 @@ void TheoryProxy::finishInit(CDCLTSatSolverInterface* ss, CnfStream* cs)
   {
     d_decisionEngine.reset(new decision::DecisionEngineEmpty(d_env));
   }
+  // make the theory preregistrar
+  d_prr.reset(new TheoryPreregistrar(d_env, d_theoryEngine, ss, cs));
   // compute if we need to track skolem definitions
-  d_trackActiveSkDefs = d_decisionEngine->needsActiveSkolemDefs();
+  d_trackActiveSkDefs = d_decisionEngine->needsActiveSkolemDefs()
+                        || d_prr->needsActiveSkolemDefs();
   d_cnfStream = cs;
 }
 
@@ -149,17 +154,31 @@ void TheoryProxy::notifySkolemDefinition(Node a, TNode skolem)
 
 void TheoryProxy::notifyAssertion(Node a, TNode skolem, bool isLemma)
 {
+  // ignore constants
+  if (a.isConst())
+  {
+    return;
+  }
   // notify the decision engine
   d_decisionEngine->addAssertion(a, skolem, isLemma);
+  // notify the preregistrar
+  d_prr->addAssertion(a, skolem, isLemma);
 }
 
 void TheoryProxy::variableNotify(SatVariable var) {
-  preRegister(getNode(SatLiteral(var)));
+  notifySatLiteral(getNode(SatLiteral(var)));
 }
 
 void TheoryProxy::theoryCheck(theory::Theory::Effort effort) {
-  while (!d_queue.empty()) {
-    TNode assertion = d_queue.front();
+  Trace("theory-proxy") << "TheoryProxy: check " << effort << std::endl;
+  d_activatedSkDefs = false;
+  // check with the preregistrar
+  d_prr->check();
+  TNode assertion;
+  int32_t alevel;
+  while (!d_queue.empty())
+  {
+    std::tie(assertion, alevel) = d_queue.front();
     d_queue.pop();
     if (d_zll != nullptr)
     {
@@ -167,20 +186,27 @@ void TheoryProxy::theoryCheck(theory::Theory::Effort effort) {
       {
         break;
       }
-      int32_t alevel = d_propEngine->getDecisionLevel(assertion);
       if (!d_zll->notifyAsserted(assertion, alevel))
       {
         d_stopSearch = true;
         break;
       }
     }
+    // notify the preregister utility, which may trigger new preregistrations
+    if (!d_prr->notifyAsserted(assertion))
+    {
+      // the preregistrar determined we should not assert this assertion, which
+      // can be the case for Boolean variables that we are notified about for
+      // the purposes of updating justification when using preregistration
+      // mode relevant.
+      continue;
+    }
     // now, assert to theory engine
+    Trace("prereg") << "assert: " << assertion << std::endl;
     d_theoryEngine->assertFact(assertion);
     if (d_trackActiveSkDefs)
     {
       Assert(d_skdm != nullptr);
-      Trace("sat-rlv-assert")
-          << "Assert to theory engine: " << assertion << std::endl;
       // Assertion makes all skolems in assertion active,
       // which triggers their definitions to becoming active.
       std::vector<TNode> activeSkolemDefs;
@@ -190,6 +216,16 @@ void TheoryProxy::theoryCheck(theory::Theory::Effort effort) {
         // notify the decision engine of the skolem definitions that have become
         // active due to the assertion.
         d_decisionEngine->notifyActiveSkolemDefs(activeSkolemDefs);
+        d_prr->notifyActiveSkolemDefs(activeSkolemDefs);
+        // if we are doing a FULL effort check (propagating with no remaining
+        // decisions) and a new skolem definition becomes active, then the SAT
+        // assignment is not complete.
+        if (effort == theory::Theory::EFFORT_FULL)
+        {
+          Trace("theory-proxy") << "...change check to STANDARD!" << std::endl;
+          effort = theory::Theory::EFFORT_STANDARD;
+        }
+        d_activatedSkDefs = true;
       }
     }
   }
@@ -265,7 +301,8 @@ void TheoryProxy::enqueueTheoryLiteral(const SatLiteral& l) {
   Node literalNode = d_cnfStream->getNode(l);
   Trace("prop") << "enqueueing theory literal " << l << " " << literalNode << std::endl;
   Assert(!literalNode.isNull());
-  d_queue.push(literalNode);
+  // Decision level = SAT context level - 1 due to global push().
+  d_queue.push(std::make_pair(literalNode, context()->getLevel() - 1));
 }
 
 SatLiteral TheoryProxy::getNextTheoryDecisionRequest() {
@@ -276,14 +313,22 @@ SatLiteral TheoryProxy::getNextTheoryDecisionRequest() {
 SatLiteral TheoryProxy::getNextDecisionEngineRequest(bool &stopSearch) {
   Assert(d_decisionEngine != NULL);
   Assert(stopSearch != true);
+  Trace("theory-proxy") << "TheoryProxy: getNextDecisionEngineRequest"
+                        << std::endl;
   if (d_stopSearch.get())
   {
+    Trace("theory-proxy") << "...stopped search, finish" << std::endl;
     stopSearch = true;
     return undefSatLiteral;
   }
   SatLiteral ret = d_decisionEngine->getNext(stopSearch);
   if(stopSearch) {
-    Trace("decision") << "  ***  Decision Engine stopped search *** " << std::endl;
+    Trace("theory-proxy") << "  ***  Decision Engine stopped search *** "
+                          << std::endl;
+  }
+  else
+  {
+    Trace("theory-proxy") << "...returned next decision" << std::endl;
   }
   return ret;
 }
@@ -293,7 +338,18 @@ bool TheoryProxy::theoryNeedCheck() const {
   {
     return false;
   }
-  return d_theoryEngine->needCheck();
+  else if (d_activatedSkDefs)
+  {
+    // a new skolem definition become active on the last call to theoryCheck,
+    // return true
+    return true;
+  }
+  // otherwise ask the theory engine, which will return true if its output
+  // channel was used.
+  bool needCheck = d_theoryEngine->needCheck();
+  Trace("theory-proxy") << "TheoryProxy: theoryNeedCheck returns " << needCheck
+                        << std::endl;
+  return needCheck;
 }
 
 bool TheoryProxy::isModelUnsound() const
@@ -378,7 +434,17 @@ void TheoryProxy::getSkolems(TNode node,
   }
 }
 
-void TheoryProxy::preRegister(Node n) { d_theoryEngine->preRegister(n); }
+void TheoryProxy::notifySatLiteral(Node n)
+{
+  // notify the preregister utility, which may trigger new preregistrations
+  d_prr->notifySatLiteral(n);
+}
+
+void TheoryProxy::notifyBacktrack(uint32_t nlevels)
+{
+  // notify the preregistrar, which may trigger reregistrations
+  d_prr->notifyBacktrack(nlevels);
+}
 
 std::vector<Node> TheoryProxy::getLearnedZeroLevelLiterals(
     modes::LearnedLitType ltype) const
