@@ -268,17 +268,6 @@ void SetDefaults::finalizeLogic(LogicInfo& logic, Options& opts) const
       throw OptionException(
           "solving bitvectors as integers is incompatible with --bool-to-bv.");
     }
-    if (opts.smt.BVAndIntegerGranularity > 8)
-    {
-      /**
-       * The granularity sets the size of the ITE in each element
-       * of the sum that is generated for bitwise operators.
-       * The size of the ITE is 2^{2*granularity}.
-       * Since we don't want to introduce ITEs with unbounded size,
-       * we bound the granularity.
-       */
-      throw OptionException("solve-bv-as-int accepts values from 0 to 8.");
-    }
     if (logic.isTheoryEnabled(THEORY_BV))
     {
       logic = logic.getUnlockedCopy();
@@ -314,24 +303,22 @@ void SetDefaults::finalizeLogic(LogicInfo& logic, Options& opts) const
     }
   }
 
-  // Set default options associated with strings-exp. We also set these options
-  // if we are using eager string preprocessing, which may introduce quantified
-  // formulas at preprocess time.
-  //
-  // We don't want to set this option when we are in logics that contain ALL.
-  //
-  // We also must enable stringExp if reElim is aggressive, since this
-  // introduces bounded quantifiers during preprocessing.
-  if ((!logic.hasEverything() && logic.isTheoryEnabled(THEORY_STRINGS))
-      || opts.strings.regExpElim == options::RegExpElimMode::AGG)
+  // Set default options associated with strings-exp, which is enabled by
+  // default if the logic includes strings. Note that enabling stringExp
+  // enables quantifiers in the logic, and enables the bounded integer
+  // quantifiers module for processing *only* bounded quantifiers generated by
+  // the strings theory. It should not have an impact otherwise.
+  if (logic.isTheoryEnabled(THEORY_STRINGS)
+      && !options().strings.stringExpWasSetByUser)
   {
-    // If the user explicitly set a logic that includes strings, but is not
-    // the generic "ALL" logic, then enable stringsExp.
     opts.writeStrings().stringExp = true;
-    Trace("smt") << "Turning stringExp on since logic does not have everything "
-                    "and string theory is enabled\n";
+    Trace("smt") << "Turning stringExp on since strings are enabled\n";
   }
-  if (opts.strings.stringExp || !opts.strings.stringLazyPreproc)
+  // If strings-exp is enabled, we require quantifiers. We also enable them
+  // if we are using eager string preprocessing or aggressive regular expression
+  // elimination, which may introduce quantified formulas at preprocess time.
+  if (opts.strings.stringExp || !opts.strings.stringLazyPreproc
+      || opts.strings.regExpElim == options::RegExpElimMode::AGG)
   {
     // We require quantifiers since extended functions reduce using them.
     if (!logic.isQuantified())
