@@ -23,6 +23,7 @@
 #include "expr/bound_var_manager.h"
 #include "expr/node.h"
 #include "expr/node_algorithm.h"
+#include "expr/subtype_elim_node_converter.h"
 #include "options/base_options.h"
 #include "options/expr_options.h"
 #include "options/language.h"
@@ -861,6 +862,18 @@ void SolverEngine::assertSygusConstraint(Node n, bool isAssume)
   d_sygusSolver->assertSygusConstraint(n, isAssume);
 }
 
+std::vector<Node> SolverEngine::getSygusConstraints()
+{
+  finishInit();
+  return d_sygusSolver->getSygusConstraints();
+}
+
+std::vector<Node> SolverEngine::getSygusAssumptions()
+{
+  finishInit();
+  return d_sygusSolver->getSygusAssumptions();
+}
+
 void SolverEngine::assertSygusInvConstraint(Node inv,
                                             Node pre,
                                             Node trans,
@@ -957,7 +970,11 @@ Node SolverEngine::simplify(const Node& t)
   // apply substitutions
   tt = d_smtSolver->getPreprocessor()->applySubstitutions(tt);
   // now rewrite
-  return d_env->getRewriter()->rewrite(tt);
+  Node ret = d_env->getRewriter()->rewrite(tt);
+  // make so that the returned term does not involve arithmetic subtyping
+  SubtypeElimNodeConverter senc;
+  ret = senc.convert(ret);
+  return ret;
 }
 
 Node SolverEngine::getValue(const Node& t) const
@@ -1183,10 +1200,18 @@ void SolverEngine::ensureWellFormedTerm(const Node& n,
     bool wasShadow = false;
     if (expr::hasFreeOrShadowedVar(n, wasShadow))
     {
-      std::string varType(wasShadow ? "shadowed" : "free");
       std::stringstream se;
-      se << "Cannot process term with " << varType << " variable in " << src
-         << ".";
+      se << "Cannot process term " << n << " with ";
+      if (wasShadow)
+      {
+        se << "shadowed variables " << std::endl;
+      }
+      else
+      {
+        std::unordered_set<internal::Node> fvs;
+        expr::getFreeVariables(n, fvs);
+        se << "free variables: " << fvs << std::endl;
+      }
       throw ModalException(se.str().c_str());
     }
   }
