@@ -188,6 +188,16 @@ RewriteResponse ArithRewriter::postRewriteAtom(TNode atom)
   rewriter::addToSum(sum, left, negate);
   rewriter::addToSum(sum, right, !negate);
 
+  if (kind != Kind::EQUAL)
+  {
+    // see if we should convert the inequality to a bitvector inequality
+    RewriteResponse rineqBv = rewriteIneqToBv(kind, sum, atom);
+    if (rineqBv.d_node != atom)
+    {
+      return rineqBv;
+    }
+  }
+
   // Now we have (sum <kind> 0)
   if (rewriter::isIntegral(sum))
   {
@@ -329,7 +339,8 @@ RewriteResponse ArithRewriter::postRewriteTerm(TNode t){
         }
         else if (t[0].isConst()
                  && t[0].getConst<Rational>().getNumerator().toUnsignedInt()
-                        == 2)
+                        == 2
+                 && t[1].getType().isInteger())
         {
           return RewriteResponse(
               REWRITE_DONE, NodeManager::currentNM()->mkNode(kind::POW2, t[1]));
@@ -805,7 +816,7 @@ RewriteResponse ArithRewriter::rewriteExtIntegerOp(TNode t)
 RewriteResponse ArithRewriter::postRewriteIAnd(TNode t)
 {
   Assert(t.getKind() == kind::IAND);
-  size_t bsize = t.getOperator().getConst<IntAnd>().d_size;
+  uint32_t bsize = t.getOperator().getConst<IntAnd>().d_size;
   NodeManager* nm = NodeManager::currentNM();
   // if constant, we eliminate
   if (t[0].isConst() && t[1].isConst())
@@ -1123,6 +1134,93 @@ RewriteResponse ArithRewriter::returnRewrite(TNode t, Node ret, Rewrite r)
   return RewriteResponse(REWRITE_AGAIN_FULL, ret);
 }
 
+RewriteResponse ArithRewriter::rewriteIneqToBv(Kind kind,
+                                               const rewriter::Sum& sum,
+                                               const Node& ineq)
+{
+  bool convertible = true;
+  // the (single) bv2nat term in the sum
+  Node bv2natTerm;
+  // whether the bv2nat term is positive in the sum
+  bool bv2natPol = false;
+  // the remaining sum (constant)
+  std::vector<Node> otherSum;
+  NodeManager* nm = NodeManager::currentNM();
+  for (const std::pair<const Node, RealAlgebraicNumber>& m : sum)
+  {
+    if (m.second.isRational())
+    {
+      const Rational& r = m.second.toRational();
+      Kind mk = m.first.getKind();
+      if (mk == BITVECTOR_TO_NAT)
+      {
+        // We currently only eliminate sums involving exactly one
+        // (bv2nat x) monomial whose coefficient is +- 1, although more
+        // cases could be handled here.
+        if (bv2natTerm.isNull())
+        {
+          if (r.abs().isOne())
+          {
+            bv2natPol = (r.sgn() == 1);
+            bv2natTerm = m.first;
+            continue;
+          }
+        }
+        else
+        {
+          convertible = false;
+          break;
+        }
+      }
+      else if (mk == CONST_INTEGER && m.second.isRational())
+      {
+        if (r.isIntegral())
+        {
+          otherSum.push_back(nm->mkConstInt(r));
+          continue;
+        }
+      }
+      // if a non-constant, non-bv2nat term is in the sum, we fail
+    }
+    convertible = false;
+    break;
+  }
+  if (convertible && !bv2natTerm.isNull())
+  {
+    Node zero = nm->mkConstInt(Rational(0));
+    Kind bvKind = (kind == GT ? (bv2natPol ? BITVECTOR_UGT : BITVECTOR_ULT)
+                              : (bv2natPol ? BITVECTOR_UGE : BITVECTOR_ULE));
+    Node bvt = bv2natTerm[0];
+    size_t bvsize = bvt.getType().getBitVectorSize();
+    Node w = nm->mkConstInt(Rational(Integer(2).pow(bvsize)));
+    Node osum =
+        otherSum.empty()
+            ? zero
+            : (otherSum.size() == 1 ? otherSum[0] : nm->mkNode(ADD, otherSum));
+    Node o = bv2natPol ? nm->mkNode(NEG, osum) : osum;
+    Node ub = nm->mkNode(GEQ, o, w);
+    Node lb = nm->mkNode(LT, o, zero);
+    Node iToBvop = nm->mkConst(IntToBitVector(bvsize));
+    Node ret = nm->mkNode(
+        ITE,
+        ub,
+        nm->mkConst(!bv2natPol),
+        nm->mkNode(
+            ITE,
+            lb,
+            nm->mkConst(bv2natPol),
+            nm->mkNode(bvKind, bvt, nm->mkNode(INT_TO_BITVECTOR, iToBvop, o))));
+    // E.g. (<= (bv2nat x) N) -->
+    //      (ite (>= N 2^w) true (ite (< N 0) false (bvule x ((_ int2bv w) N))
+    // or   (<= N (bv2nat x)) -->
+    //      (ite (>= N 2^w) false (ite (< N 0) true (bvuge x ((_ int2bv w) N))
+    // where N is a constant. Note that ((_ int2bv w) N) will subsequently
+    // be rewritten to the appropriate bitvector constant.
+    return returnRewrite(ineq, ret, Rewrite::INEQ_BV_TO_NAT_ELIM);
+  }
+  return RewriteResponse(REWRITE_DONE, ineq);
+}
+
 }  // namespace arith
 }  // namespace theory
 }  // namespace cvc5::internal
