@@ -28,6 +28,7 @@
 #include "theory/ext_theory.h"
 #include "theory/rewriter.h"
 #include "theory/theory_model.h"
+#include "util/cocoa_globals.h"
 
 using namespace std;
 using namespace cvc5::internal::kind;
@@ -52,6 +53,10 @@ TheoryArith::TheoryArith(Env& env, OutputChannel& out, Valuation valuation)
       d_rewriter(d_opElim),
       d_arithModelCacheSet(false)
 {
+#ifdef CVC5_USE_COCOA
+  // must be initialized before using CoCoA.
+  initCocoaGlobalManager();
+#endif /* CVC5_USE_COCOA */
   // currently a cyclic dependency to TheoryArithPrivate
   d_astate.setParent(d_internal);
   // indicate we are using the theory state object and inference manager
@@ -436,14 +441,15 @@ EqualityStatus TheoryArith::getEqualityStatus(TNode a, TNode b) {
   return EQUALITY_UNKNOWN;
 }
 
-Node TheoryArith::getModelValue(TNode var) {
+Node TheoryArith::getCandidateModelValue(TNode var)
+{
   var = var.getKind() == kind::TO_REAL ? var[0] : var;
   std::map<Node, Node>::iterator it = d_arithModelCache.find(var);
   if (it != d_arithModelCache.end())
   {
     return it->second;
   }
-  return d_internal->getModelValue( var );
+  return d_internal->getCandidateModelValue(var);
 }
 
 std::pair<bool, Node> TheoryArith::entailmentCheck(TNode lit)
@@ -514,11 +520,14 @@ bool TheoryArith::sanityCheckIntegerModel()
                  "integer variable "
               << p.first << " : " << p.second << std::endl;
     // must branch and bound
-    TrustNode lem =
+    std::vector<TrustNode> lems =
         d_bab.branchIntegerVariable(p.first, p.second.getConst<Rational>());
-    if (d_im.trustedLemma(lem, InferenceId::ARITH_BB_LEMMA))
+    for (const TrustNode& lem : lems)
     {
-      addedLemma = true;
+      if (d_im.trustedLemma(lem, InferenceId::ARITH_BB_LEMMA))
+      {
+        addedLemma = true;
+      }
     }
     badAssignment = true;
   }
