@@ -35,7 +35,8 @@ using namespace datatypes;
 
 TypeNode BinaryOperatorTypeRule::computeType(NodeManager* nodeManager,
                                              TNode n,
-                                             bool check)
+                                             bool check,
+                                             std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_UNION_MAX
          || n.getKind() == kind::BAG_UNION_DISJOINT
@@ -73,7 +74,8 @@ bool BinaryOperatorTypeRule::computeIsConst(NodeManager* nodeManager, TNode n)
 
 TypeNode SubBagTypeRule::computeType(NodeManager* nodeManager,
                                      TNode n,
-                                     bool check)
+                                     bool check,
+                                     std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_SUBBAG);
   TypeNode bagType = n[0].getType(check);
@@ -95,7 +97,8 @@ TypeNode SubBagTypeRule::computeType(NodeManager* nodeManager,
 
 TypeNode CountTypeRule::computeType(NodeManager* nodeManager,
                                     TNode n,
-                                    bool check)
+                                    bool check,
+                                    std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_COUNT);
   TypeNode bagType = n[1].getType(check);
@@ -124,7 +127,8 @@ TypeNode CountTypeRule::computeType(NodeManager* nodeManager,
 
 TypeNode MemberTypeRule::computeType(NodeManager* nodeManager,
                                      TNode n,
-                                     bool check)
+                                     bool check,
+                                     std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_MEMBER);
   TypeNode bagType = n[1].getType(check);
@@ -153,7 +157,8 @@ TypeNode MemberTypeRule::computeType(NodeManager* nodeManager,
 
 TypeNode DuplicateRemovalTypeRule::computeType(NodeManager* nodeManager,
                                                TNode n,
-                                               bool check)
+                                               bool check,
+                                               std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_DUPLICATE_REMOVAL);
   TypeNode bagType = n[0].getType(check);
@@ -170,7 +175,10 @@ TypeNode DuplicateRemovalTypeRule::computeType(NodeManager* nodeManager,
   return bagType;
 }
 
-TypeNode BagMakeTypeRule::computeType(NodeManager* nm, TNode n, bool check)
+TypeNode BagMakeTypeRule::computeType(NodeManager* nm,
+                                      TNode n,
+                                      bool check,
+                                      std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_MAKE);
   TypeNode actualElementType = n[0].getType(check);
@@ -206,7 +214,8 @@ bool BagMakeTypeRule::computeIsConst(NodeManager* nodeManager, TNode n)
 
 TypeNode IsSingletonTypeRule::computeType(NodeManager* nodeManager,
                                           TNode n,
-                                          bool check)
+                                          bool check,
+                                          std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_IS_SINGLETON);
   TypeNode bagType = n[0].getType(check);
@@ -223,7 +232,8 @@ TypeNode IsSingletonTypeRule::computeType(NodeManager* nodeManager,
 
 TypeNode EmptyBagTypeRule::computeType(NodeManager* nodeManager,
                                        TNode n,
-                                       bool check)
+                                       bool check,
+                                       std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_EMPTY);
   EmptyBag emptyBag = n.getConst<EmptyBag>();
@@ -232,7 +242,8 @@ TypeNode EmptyBagTypeRule::computeType(NodeManager* nodeManager,
 
 TypeNode CardTypeRule::computeType(NodeManager* nodeManager,
                                    TNode n,
-                                   bool check)
+                                   bool check,
+                                   std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_CARD);
   TypeNode bagType = n[0].getType(check);
@@ -249,7 +260,8 @@ TypeNode CardTypeRule::computeType(NodeManager* nodeManager,
 
 TypeNode ChooseTypeRule::computeType(NodeManager* nodeManager,
                                      TNode n,
-                                     bool check)
+                                     bool check,
+                                     std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_CHOOSE);
   TypeNode bagType = n[0].getType(check);
@@ -266,7 +278,8 @@ TypeNode ChooseTypeRule::computeType(NodeManager* nodeManager,
 
 TypeNode FromSetTypeRule::computeType(NodeManager* nodeManager,
                                       TNode n,
-                                      bool check)
+                                      bool check,
+                                      std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_FROM_SET);
   TypeNode setType = n[0].getType(check);
@@ -285,7 +298,8 @@ TypeNode FromSetTypeRule::computeType(NodeManager* nodeManager,
 
 TypeNode ToSetTypeRule::computeType(NodeManager* nodeManager,
                                     TNode n,
-                                    bool check)
+                                    bool check,
+                                    std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_TO_SET);
   TypeNode bagType = n[0].getType(check);
@@ -304,7 +318,8 @@ TypeNode ToSetTypeRule::computeType(NodeManager* nodeManager,
 
 TypeNode BagMapTypeRule::computeType(NodeManager* nodeManager,
                                      TNode n,
-                                     bool check)
+                                     bool check,
+                                     std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_MAP);
   TypeNode functionType = n[0].getType(check);
@@ -346,7 +361,8 @@ TypeNode BagMapTypeRule::computeType(NodeManager* nodeManager,
 
 TypeNode BagFilterTypeRule::computeType(NodeManager* nodeManager,
                                         TNode n,
-                                        bool check)
+                                        bool check,
+                                        std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_FILTER);
   TypeNode functionType = n[0].getType(check);
@@ -388,7 +404,8 @@ TypeNode BagFilterTypeRule::computeType(NodeManager* nodeManager,
 
 TypeNode BagFoldTypeRule::computeType(NodeManager* nodeManager,
                                       TNode n,
-                                      bool check)
+                                      bool check,
+                                      std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_FOLD);
   TypeNode functionType = n[0].getType(check);
@@ -439,7 +456,8 @@ TypeNode BagFoldTypeRule::computeType(NodeManager* nodeManager,
 
 TypeNode BagPartitionTypeRule::computeType(NodeManager* nodeManager,
                                            TNode n,
-                                           bool check)
+                                           bool check,
+                                           std::ostream* errOut)
 {
   Assert(n.getKind() == kind::BAG_PARTITION);
   TypeNode functionType = n[0].getType(check);
@@ -483,7 +501,8 @@ TypeNode BagPartitionTypeRule::computeType(NodeManager* nodeManager,
 
 TypeNode TableProductTypeRule::computeType(NodeManager* nodeManager,
                                            TNode n,
-                                           bool check)
+                                           bool check,
+                                           std::ostream* errOut)
 {
   Assert(n.getKind() == kind::TABLE_PRODUCT);
   Node A = n[0];
@@ -518,7 +537,10 @@ TypeNode TableProductTypeRule::computeType(NodeManager* nodeManager,
   return retType;
 }
 
-TypeNode TableProjectTypeRule::computeType(NodeManager* nm, TNode n, bool check)
+TypeNode TableProjectTypeRule::computeType(NodeManager* nm,
+                                           TNode n,
+                                           bool check,
+                                           std::ostream* errOut)
 {
   Assert(n.getKind() == kind::TABLE_PROJECT && n.hasOperator()
          && n.getOperator().getKind() == kind::TABLE_PROJECT_OP);
@@ -575,7 +597,8 @@ TypeNode TableProjectTypeRule::computeType(NodeManager* nm, TNode n, bool check)
 
 TypeNode TableAggregateTypeRule::computeType(NodeManager* nm,
                                              TNode n,
-                                             bool check)
+                                             bool check,
+                                             std::ostream* errOut)
 {
   Assert(n.getKind() == kind::TABLE_AGGREGATE && n.hasOperator()
          && n.getOperator().getKind() == kind::TABLE_AGGREGATE_OP);
@@ -639,7 +662,10 @@ TypeNode TableAggregateTypeRule::computeType(NodeManager* nm,
   return nm->mkBagType(functionType.getRangeType());
 }
 
-TypeNode TableJoinTypeRule::computeType(NodeManager* nm, TNode n, bool check)
+TypeNode TableJoinTypeRule::computeType(NodeManager* nm,
+                                        TNode n,
+                                        bool check,
+                                        std::ostream* errOut)
 {
   Assert(n.getKind() == kind::TABLE_JOIN && n.hasOperator()
          && n.getOperator().getKind() == kind::TABLE_JOIN_OP);
@@ -706,7 +732,10 @@ TypeNode TableJoinTypeRule::computeType(NodeManager* nm, TNode n, bool check)
   return nm->mkBagType(retTupleType);
 }
 
-TypeNode TableGroupTypeRule::computeType(NodeManager* nm, TNode n, bool check)
+TypeNode TableGroupTypeRule::computeType(NodeManager* nm,
+                                         TNode n,
+                                         bool check,
+                                         std::ostream* errOut)
 {
   Assert(n.getKind() == kind::TABLE_GROUP && n.hasOperator()
          && n.getOperator().getKind() == kind::TABLE_GROUP_OP);
