@@ -166,6 +166,7 @@ enum RewriteRuleId
   UgtUrem,
 
   UltOne,
+  UltOnes,
   SltZero,
   ZeroUlt,
   MergeSignExtend,
@@ -173,6 +174,7 @@ enum RewriteRuleId
   ZeroExtendEqConst,
   SignExtendUltConst,
   ZeroExtendUltConst,
+  IneqElimConversion,
 
   /// normalization rules
   ExtractBitwise,
@@ -357,6 +359,7 @@ inline std::ostream& operator << (std::ostream& out, RewriteRuleId ruleId) {
   case NegAdd: out << "NegAdd"; return out;
   case BBAddNeg: out << "BBAddNeg"; return out;
   case UltOne : out << "UltOne"; return out;
+  case UltOnes: out << "UltOnes"; return out;
   case SltZero : out << "SltZero"; return out;
   case ZeroUlt : out << "ZeroUlt"; return out;
   case MergeSignExtend : out << "MergeSignExtend"; return out;
@@ -364,7 +367,8 @@ inline std::ostream& operator << (std::ostream& out, RewriteRuleId ruleId) {
   case ZeroExtendEqConst: out << "ZeroExtendEqConst"; return out;
   case SignExtendUltConst: out << "SignExtendUltConst"; return out;
   case ZeroExtendUltConst: out << "ZeroExtendUltConst"; return out;
-    
+  case IneqElimConversion: out << "IneqElimConversion"; return out;
+
   case UleEliminate : out << "UleEliminate"; return out;
   case BitwiseSlicing : out << "BitwiseSlicing"; return out;
   case ExtractSignExtend : out << "ExtractSignExtend"; return out;
@@ -614,6 +618,7 @@ struct AllRewriteRules {
   RewriteRule<SremEliminate> rule144;
   RewriteRule<SmodEliminate> rule145;
   RewriteRule<UgtUrem> rule146;
+  RewriteRule<UltOnes> rule147;
 };
 
 template<> inline
