@@ -59,12 +59,15 @@ RewriteResponse TheoryBVRewriter::RewriteBitOf(TNode node, bool prerewrite)
 
 RewriteResponse TheoryBVRewriter::RewriteUlt(TNode node, bool prerewrite) {
   // reduce common subexpressions on both sides
-  Node resultNode = LinearRewriteStrategy
-    < RewriteRule<EvalUlt>, // if both arguments are constants evaluates
-      RewriteRule<UltZero>, // a < 0 rewrites to false,
-      RewriteRule<SignExtendUltConst>,
-      RewriteRule<ZeroExtendUltConst>
-       >::apply(node);
+  Node resultNode =
+      LinearRewriteStrategy<RewriteRule<EvalUlt>,  // if both arguments are
+                                                   // constants evaluates
+                            RewriteRule<UltOne>,
+                            RewriteRule<UltOnes>,
+                            RewriteRule<UltZero>,  // a < 0 rewrites to false,
+                            RewriteRule<SignExtendUltConst>,
+                            RewriteRule<ZeroExtendUltConst>,
+                            RewriteRule<IneqElimConversion>>::apply(node);
 
   return RewriteResponse(resultNode == node ? REWRITE_DONE : REWRITE_AGAIN_FULL,
                          resultNode);
@@ -105,14 +108,14 @@ RewriteResponse TheoryBVRewriter::RewriteSltBv(TNode node, bool prerewrite){
 }
 
 RewriteResponse TheoryBVRewriter::RewriteUle(TNode node, bool prerewrite){
-  Node resultNode = LinearRewriteStrategy
-    < RewriteRule<EvalUle>,
-      RewriteRule<UleMax>,
-      RewriteRule<ZeroUle>,
-      RewriteRule<UleZero>,
-      RewriteRule<UleSelf>,
-      RewriteRule<UleEliminate>
-      >::apply(node);
+  Node resultNode =
+      LinearRewriteStrategy<RewriteRule<EvalUle>,
+                            RewriteRule<UleMax>,
+                            RewriteRule<ZeroUle>,
+                            RewriteRule<IneqElimConversion>,
+                            RewriteRule<UleZero>,
+                            RewriteRule<UleSelf>,
+                            RewriteRule<UleEliminate>>::apply(node);
   return RewriteResponse(resultNode == node ? REWRITE_DONE : REWRITE_AGAIN,
                          resultNode);
 }
