@@ -25,21 +25,33 @@ namespace cvc5::internal {
 namespace theory {
 
 EngineOutputChannel::Statistics::Statistics(StatisticsRegistry& sr,
-                                            theory::TheoryId theory)
-    : conflicts(sr.registerInt(getStatsPrefix(theory) + "conflicts")),
-      propagations(sr.registerInt(getStatsPrefix(theory) + "propagations")),
-      lemmas(sr.registerInt(getStatsPrefix(theory) + "lemmas")),
-      requirePhase(sr.registerInt(getStatsPrefix(theory) + "requirePhase")),
-      trustedConflicts(
-          sr.registerInt(getStatsPrefix(theory) + "trustedConflicts")),
-      trustedLemmas(sr.registerInt(getStatsPrefix(theory) + "trustedLemmas"))
+                                            const std::string& statPrefix)
+    : conflicts(sr.registerInt(statPrefix + "conflicts")),
+      propagations(sr.registerInt(statPrefix + "propagations")),
+      lemmas(sr.registerInt(statPrefix + "lemmas")),
+      requirePhase(sr.registerInt(statPrefix + "requirePhase")),
+      trustedConflicts(sr.registerInt(statPrefix + "trustedConflicts")),
+      trustedLemmas(sr.registerInt(statPrefix + "trustedLemmas"))
 {
 }
 
 EngineOutputChannel::EngineOutputChannel(StatisticsRegistry& sr,
                                          TheoryEngine* engine,
                                          theory::TheoryId theory)
-    : d_engine(engine), d_statistics(sr, theory), d_theory(theory)
+    : d_engine(engine),
+      d_name(toString(theory)),
+      d_statistics(sr, getStatsPrefix(theory)),
+      d_theory(theory)
+{
+}
+
+EngineOutputChannel::EngineOutputChannel(StatisticsRegistry& sr,
+                                         TheoryEngine* engine,
+                                         const std::string& name)
+    : d_engine(engine),
+      d_name(name),
+      d_statistics(sr, name + "::"),
+      d_theory(THEORY_NONE)
 {
 }
 
