@@ -44,9 +44,14 @@ class EngineOutputChannel : public theory::OutputChannel
   friend class internal::TheoryEngine;
 
  public:
+  /** Constructor for use by theory */
   EngineOutputChannel(StatisticsRegistry& sr,
                       TheoryEngine* engine,
                       theory::TheoryId theory);
+  /** Constructor for use by non-theory */
+  EngineOutputChannel(StatisticsRegistry& sr,
+                      TheoryEngine* engine,
+                      const std::string& name);
 
   void safePoint(Resource r) override;
 
@@ -86,22 +91,19 @@ class EngineOutputChannel : public theory::OutputChannel
   class Statistics
   {
    public:
-    Statistics(StatisticsRegistry& sr, theory::TheoryId theory);
+    Statistics(StatisticsRegistry& sr, const std::string& statPrefix);
     /** Number of calls to conflict, propagate, lemma, requirePhase */
     IntStat conflicts, propagations, lemmas, requirePhase, trustedConflicts,
         trustedLemmas;
   };
   /** The theory engine we're communicating with. */
   TheoryEngine* d_engine;
+  /** The name of the owner of this channel. */
+  std::string d_name;
   /** The statistics of the theory interractions. */
   Statistics d_statistics;
   /** The theory owning this channel. */
   theory::TheoryId d_theory;
-  /** A helper function for registering lemma recipes with the proof engine */
-  void registerLemmaRecipe(Node lemma,
-                           Node originalLemma,
-                           bool preprocess,
-                           theory::TheoryId theoryId);
 };
 
 }  // namespace theory
