@@ -410,6 +410,8 @@ void TermDbSygus::registerEnumerator(Node e,
   std::vector<TypeNode> sf_types;
   eti.getSubfieldTypes(sf_types);
   bool sharedSel = options().datatypes.dtSharedSelectors;
+  // whether this enumerator relies on any-constant constructors
+  bool usingAnyConst = false;
   // for each type of subfield type of this enumerator
   for (unsigned i = 0, ntypes = sf_types.size(); i < ntypes; i++)
   {
@@ -419,17 +421,21 @@ void TermDbSygus::registerEnumerator(Node e,
     SygusTypeInfo& sti = getTypeInfo(stn);
     const DType& dt = stn.getDType();
     int anyC = sti.getAnyConstantConsNum();
-    for (unsigned j = 0, ncons = dt.getNumConstructors(); j < ncons; j++)
+    if (anyC != -1)
     {
-      bool isAnyC = static_cast<int>(j) == anyC;
-      if (anyC != -1 && !isAnyC)
+      usingAnyConst = true;
+      for (unsigned j = 0, ncons = dt.getNumConstructors(); j < ncons; j++)
       {
-        // if we are using the any constant constructor, do not use any
-        // concrete constant
-        Node c_op = sti.getConsNumConst(j);
-        if (!c_op.isNull())
+        bool isAnyC = static_cast<int>(j) == anyC;
+        if (!isAnyC)
         {
-          rm_indices.push_back(j);
+          // if we are using the any constant constructor, do not use any
+          // concrete constant
+          Node c_op = sti.getConsNumConst(j);
+          if (!c_op.isNull())
+          {
+            rm_indices.push_back(j);
+          }
         }
       }
     }
@@ -540,6 +546,19 @@ void TermDbSygus::registerEnumerator(Node e,
       isActiveGen = false;
     }
   }
+  // When we are using smart enumeration, we often do not consider model
+  // values for arguments of any-constant constructors (in sygus_explain.cpp),
+  // hence those blocking lemmas are refutation unsound. For simplicity, we
+  // mark unsound once and for all at the beginning, meaning we do not
+  // answer "infeasible" when using smart enuemration + any-constant
+  // constructors. Using --sygus-repair-const on the other hand avoids this
+  // incompleteness, which is checked here.
+  if (!isActiveGen && usingAnyConst && !options().quantifiers.sygusRepairConst)
+  {
+    Assert(d_qim != nullptr);
+    d_qim->setRefutationUnsound(
+        IncompleteId::QUANTIFIERS_SYGUS_SMART_BLOCK_ANY_CONSTANT);
+  }
   d_enum_active_gen[e] = isActiveGen;
   d_enum_basic[e] = isActiveGen && !isVarAgnostic;
 
