@@ -18,6 +18,7 @@
 #include "expr/attribute.h"
 #include "expr/dtype.h"
 #include "expr/dtype_cons.h"
+#include "expr/elim_shadow_converter.h"
 #include "options/sets_options.h"
 #include "theory/datatypes/tuple_utils.h"
 #include "theory/sets/normal_form.h"
@@ -327,6 +328,8 @@ RewriteResponse TheorySetsRewriter::postRewrite(TNode node) {
     break;
   }  // kind::SET_IS_SINGLETON
 
+  case SET_COMPREHENSION: return postRewriteComprehension(node); break;
+
   case SET_MAP: return postRewriteMap(node);
   case SET_FILTER: return postRewriteFilter(node);
   case SET_FOLD: return postRewriteFold(node);
@@ -634,6 +637,16 @@ RewriteResponse TheorySetsRewriter::preRewrite(TNode node) {
   return RewriteResponse(REWRITE_DONE, node);
 }
 
+RewriteResponse TheorySetsRewriter::postRewriteComprehension(TNode n)
+{
+  Node ne = ElimShadowNodeConverter::eliminateShadow(n);
+  if (ne != n)
+  {
+    return RewriteResponse(REWRITE_AGAIN_FULL, ne);
+  }
+  return RewriteResponse(REWRITE_DONE, n);
+}
+
 RewriteResponse TheorySetsRewriter::postRewriteMap(TNode n)
 {
   Assert(n.getKind() == kind::SET_MAP);
