@@ -629,8 +629,9 @@ bool ArithEntail::checkWithEqAssumption(Node assumption, Node a, bool strict)
   Trace("strings-entail") << "checkWithEqAssumption: subs " << v << " -> "
                           << solution << std::endl;
 
-  // use capture avoiding substitution
-  a = expr::substituteCaptureAvoiding(a, v, solution);
+  TNode tv = v;
+  TNode tsolution = solution;
+  a = a.substitute(tv, tsolution);
   return check(a, strict);
 }
 
