@@ -532,17 +532,20 @@ bool Theory::areCareDisequal(TNode x, TNode y)
   {
     return true;
   }
+  // first just check if they are disequal, which is sufficient for
+  // non-shared terms.
+  if (d_equalityEngine->areDisequal(x, y, false))
+  {
+    return true;
+  }
+  // if we aren't shared, we are not disequal
   if (!d_equalityEngine->isTriggerTerm(x, d_id)
       || !d_equalityEngine->isTriggerTerm(y, d_id))
   {
-    // just check if they are disequal, which is the used in the case for
-    // non-shared terms.
-    if (d_equalityEngine->areDisequal(x, y, false))
-    {
-      return true;
-    }
     return false;
   }
+  // otherwise use getEqualityStatus to ask the appropriate theory whether
+  // x and y are disequal.
   TNode x_shared = d_equalityEngine->getTriggerTermRepresentative(x, d_id);
   TNode y_shared = d_equalityEngine->getTriggerTermRepresentative(y, d_id);
   EqualityStatus eqStatus = d_valuation.getEqualityStatus(x_shared, y_shared);
@@ -551,7 +554,6 @@ bool Theory::areCareDisequal(TNode x, TNode y)
   {
     return true;
   }
-  Assert(!d_equalityEngine->areDisequal(x, y, false));
   return false;
 }
 
