@@ -444,12 +444,14 @@ class Theory : protected EnvObj
   virtual EqualityStatus getEqualityStatus(TNode a, TNode b);
 
   /**
-   * Return the model value of the give shared term (or null if not
-   * available).
-   *
-   * TODO (project #39): this method is likely to become deprecated.
+   * Return the candidate model value of the give shared term (or null if not
+   * available). A candidate model value is one computed at full effort,
+   * prior to running theory combination and final model construction.
+   * Typically only non-parametric theories are able to implement this method,
+   * since model construction for parametric theories involves running final
+   * model construction.
    */
-  virtual Node getModelValue(TNode var) { return Node::null(); }
+  virtual Node getCandidateModelValue(TNode var) { return Node::null(); }
 
   /** T-propagate new literal assignments in the current context. */
   virtual void propagate(Effort level = EFFORT_FULL) {}
