@@ -129,6 +129,7 @@ std::string getTheoryString(theory::TheoryId id)
 
 void TheoryEngine::finishInit()
 {
+  d_modules.clear();
   Trace("theory") << "Begin TheoryEngine::finishInit" << std::endl;
   // NOTE: This seems to be required since
   // theory::TheoryTraits<THEORY>::isParametric cannot be accessed without
@@ -160,7 +161,8 @@ void TheoryEngine::finishInit()
   // create the relevance filter if any option requires it
   if (options().theory.relevanceFilter || options().smt.produceDifficulty)
   {
-    d_relManager.reset(new RelevanceManager(d_env, Valuation(this)));
+    d_relManager.reset(new RelevanceManager(d_env, this));
+    d_modules.push_back(d_relManager.get());
   }
 
   // initialize the quantifiers engine
@@ -210,6 +212,7 @@ void TheoryEngine::finishInit()
   {
     d_partitionGen =
         std::make_unique<PartitionGenerator>(d_env, this, getPropEngine());
+    d_modules.push_back(d_partitionGen.get());
   }
   Trace("theory") << "End TheoryEngine::finishInit" << std::endl;
 }
@@ -370,16 +373,20 @@ void TheoryEngine::check(Theory::Effort effort) {
 #ifdef CVC5_FOR_EACH_THEORY_STATEMENT
 #undef CVC5_FOR_EACH_THEORY_STATEMENT
 #endif
-#define CVC5_FOR_EACH_THEORY_STATEMENT(THEORY)                      \
-  if (theory::TheoryTraits<THEORY>::hasCheck                        \
-      && isTheoryEnabled(THEORY))                       \
-  {                                                                 \
-    theoryOf(THEORY)->check(effort);                                \
-    if (d_inConflict)                                               \
-    {                                                               \
-      Trace("conflict") << THEORY << " in conflict. " << std::endl; \
-      break;                                                        \
-    }                                                               \
+#define CVC5_FOR_EACH_THEORY_STATEMENT(THEORY)                           \
+  if (theory::TheoryTraits<THEORY>::hasCheck && isTheoryEnabled(THEORY)) \
+  {                                                                      \
+    theoryOf(THEORY)->check(effort);                                     \
+    if (d_inConflict)                                                    \
+    {                                                                    \
+      Trace("conflict") << THEORY << " in conflict. " << std::endl;      \
+      break;                                                             \
+    }                                                                    \
+    if (rm->out())                                                       \
+    {                                                                    \
+      interrupt();                                                       \
+      return;                                                            \
+    }                                                                    \
   }
 
   // Do the checking
@@ -397,24 +404,17 @@ void TheoryEngine::check(Theory::Effort effort) {
     // If in full effort, we have a fake new assertion just to jumpstart the checking
     if (Theory::fullEffort(effort)) {
       d_factsAsserted = true;
-      // Reset round for the relevance manager, which notice only sets a flag
-      // to indicate that its information must be recomputed.
-      if (d_relManager != nullptr)
-      {
-        d_relManager->beginRound();
-      }
       d_tc->resetRound();
     }
 
-    if (d_partitionGen != nullptr)
+    // check with the theory modules
+    for (TheoryEngineModule* tem : d_modules)
     {
-      TrustNode tl = d_partitionGen->check(effort);
-      if (!tl.isNull())
-      {
-        lemma(tl, LemmaProperty::NONE, THEORY_LAST);
-      }
+      tem->check(effort);
     }
 
+    auto rm = d_env.getResourceManager();
+
     // Check until done
     while (d_factsAsserted && !d_inConflict && !d_lemmasAdded) {
 
@@ -443,6 +443,13 @@ void TheoryEngine::check(Theory::Effort effort) {
       // We are still satisfiable, propagate as much as possible
       propagate(effort);
 
+      // Interrupt in case we reached a resource limit.
+      if (rm->out())
+      {
+        interrupt();
+        return;
+      }
+
       if (Theory::fullEffort(effort))
       {
         d_stats.d_fullEffortChecks++;
@@ -470,6 +477,13 @@ void TheoryEngine::check(Theory::Effort effort) {
         Assert(effort == Theory::EFFORT_STANDARD);
         d_stats.d_stdEffortChecks++;
       }
+
+      // Interrupt in case we reached a resource limit.
+      if (rm->out())
+      {
+        interrupt();
+        return;
+      }
     }
 
     // Must consult quantifiers theory for last call to ensure sat, or otherwise add a lemma
@@ -482,12 +496,15 @@ void TheoryEngine::check(Theory::Effort effort) {
       // reset the model in the combination engine
       d_tc->resetModel();
       //checks for theories requiring the model go at last call
-      for (TheoryId theoryId = THEORY_FIRST; theoryId < THEORY_LAST; ++theoryId) {
-        if( theoryId!=THEORY_QUANTIFIERS ){
+      for (TheoryId theoryId = THEORY_FIRST; theoryId < THEORY_LAST; ++theoryId)
+      {
+        if (theoryId != THEORY_QUANTIFIERS)
+        {
           Theory* theory = d_theoryTable[theoryId];
           if (theory && isTheoryEnabled(theoryId))
           {
-            if( theory->needsCheckLastEffort() ){
+            if (theory->needsCheckLastEffort())
+            {
               if (!d_tc->buildModel())
               {
                 break;
@@ -505,22 +522,23 @@ void TheoryEngine::check(Theory::Effort effort) {
           d_quantEngine->check(Theory::EFFORT_LAST_CALL);
         }
       }
-      // notify the relevant manager
-      if (d_relManager != nullptr)
+      // notify the theory modules of the model
+      for (TheoryEngineModule* tem : d_modules)
       {
-        d_relManager->notifyCandidateModel(getModel());
+        tem->notifyCandidateModel(getModel());
       }
     }
 
     Trace("theory") << "TheoryEngine::check(" << effort << "): done, we are " << (d_inConflict ? "unsat" : "sat") << (d_lemmasAdded ? " with new lemmas" : " with no new lemmas");
     Trace("theory") << ", need check = " << (needCheck() ? "YES" : "NO") << endl;
 
+    // post check with the theory modules
+    for (TheoryEngineModule* tem : d_modules)
+    {
+      tem->postCheck(effort);
+    }
     if (Theory::fullEffort(effort))
     {
-      if (d_relManager != nullptr)
-      {
-        d_relManager->endRound();
-      }
       if (!d_inConflict && !needCheck())
       {
         // Do post-processing of model from the theories (e.g. used for
@@ -678,6 +696,12 @@ bool TheoryEngine::presolve() {
   } catch(const theory::Interrupted&) {
     Trace("theory") << "TheoryEngine::presolve() => interrupted" << endl;
   }
+  // presolve with the theory engine modules as well
+  for (TheoryEngineModule* tem : d_modules)
+  {
+    tem->presolve();
+  }
+
   // return whether we have a conflict
   return false;
 }/* TheoryEngine::presolve() */
@@ -725,6 +749,25 @@ void TheoryEngine::ppStaticLearn(TNode in, NodeBuilder& learned)
   CVC5_FOR_EACH_THEORY;
 }
 
+bool TheoryEngine::hasSatValue(TNode n, bool& value) const
+{
+  if (d_propEngine->isSatLiteral(n))
+  {
+    return d_propEngine->hasValue(n, value);
+  }
+  return false;
+}
+
+bool TheoryEngine::hasSatValue(TNode n) const
+{
+  if (d_propEngine->isSatLiteral(n))
+  {
+    bool value;
+    return d_propEngine->hasValue(n, value);
+  }
+  return false;
+}
+
 bool TheoryEngine::isRelevant(Node lit) const
 {
   if (d_relManager != nullptr)
@@ -746,13 +789,21 @@ theory::Theory::PPAssertStatus TheoryEngine::solve(
   TNode atom = literal.getKind() == kind::NOT ? literal[0] : literal;
   Trace("theory::solve") << "TheoryEngine::solve(" << literal << "): solving with " << theoryOf(atom)->getId() << endl;
 
-  // This should be implied by the check during ppRewrite, in particular
-  // literal should have been passed to ppRewrite.
-  Assert(isTheoryEnabled(d_env.theoryOf(atom))
-         || d_env.theoryOf(atom) == THEORY_SAT_SOLVER);
+  TheoryId tid = d_env.theoryOf(atom);
+  // Note that ppAssert is called before ppRewrite.
+  if (!isTheoryEnabled(tid) && tid != THEORY_SAT_SOLVER)
+  {
+    stringstream ss;
+    ss << "The logic was specified as " << logicInfo().getLogicString()
+       << ", which doesn't include " << tid
+       << ", but got a theory atom for that theory." << std::endl
+       << "The atom:" << std::endl
+       << atom;
+    throw LogicException(ss.str());
+  }
 
   Theory::PPAssertStatus solveStatus =
-      theoryOf(atom)->ppAssert(tliteral, substitutionOut);
+      d_theoryTable[tid]->ppAssert(tliteral, substitutionOut);
   Trace("theory::solve") << "TheoryEngine::solve(" << literal << ") => " << solveStatus << endl;
   return solveStatus;
 }
@@ -762,7 +813,7 @@ TrustNode TheoryEngine::ppRewrite(TNode term,
 {
   Assert(lems.empty());
   TheoryId tid = d_env.theoryOf(term);
-  // We check whether the theory is enabled here (instead of during solve),
+  // We check whether the theory is enabled here (instead of only during solve),
   // since there are corner cases where facts may involve terms that belong
   // to other theories, e.g. equalities between variables belong to UF when
   // theoryof-mode is `term`.
@@ -1110,7 +1161,8 @@ theory::IncompleteId TheoryEngine::getRefutationUnsoundId() const
   return d_refutationUnsoundId.get();
 }
 
-Node TheoryEngine::getModelValue(TNode var) {
+Node TheoryEngine::getCandidateModelValue(TNode var)
+{
   if (var.isConst())
   {
     // the model value of a constant must be itself
@@ -1118,7 +1170,7 @@ Node TheoryEngine::getModelValue(TNode var) {
   }
   Assert(d_sharedSolver->isShared(var))
       << "node " << var << " is not shared" << std::endl;
-  return theoryOf(d_env.theoryOf(var.getType()))->getModelValue(var);
+  return theoryOf(d_env.theoryOf(var.getType()))->getCandidateModelValue(var);
 }
 
 std::unordered_set<TNode> TheoryEngine::getRelevantAssertions(bool& success)
@@ -1142,13 +1194,14 @@ TrustNode TheoryEngine::getExplanation(TNode node)
   TNode atom = polarity ? node : node[0];
 
   // If we're not in shared mode, explanations are simple
+  TrustNode texplanation;
   if (!logicInfo().isSharingEnabled())
   {
     Trace("theory::explain")
         << "TheoryEngine::getExplanation: sharing is NOT enabled. "
         << " Responsible theory is: " << theoryOf(atom)->getId() << std::endl;
 
-    TrustNode texplanation = theoryOf(atom)->explain(node);
+    texplanation = theoryOf(atom)->explain(node);
     Node explanation = texplanation.getNode();
     Trace("theory::explain") << "TheoryEngine::getExplanation(" << node
                              << ") => " << explanation << endl;
@@ -1167,28 +1220,36 @@ TrustNode TheoryEngine::getExplanation(TNode node)
             TrustNode::mkTrustPropExp(node, explanation, d_lazyProof.get());
       }
     }
-    return texplanation;
   }
+  else
+  {
+    Trace("theory::explain")
+        << "TheoryEngine::getExplanation: sharing IS enabled" << std::endl;
 
-  Trace("theory::explain") << "TheoryEngine::getExplanation: sharing IS enabled"
-                           << std::endl;
+    // Initial thing to explain
+    NodeTheoryPair toExplain(
+        node, THEORY_SAT_SOLVER, d_propagationMapTimestamp);
+    Assert(d_propagationMap.find(toExplain) != d_propagationMap.end());
 
-  // Initial thing to explain
-  NodeTheoryPair toExplain(node, THEORY_SAT_SOLVER, d_propagationMapTimestamp);
-  Assert(d_propagationMap.find(toExplain) != d_propagationMap.end());
+    NodeTheoryPair nodeExplainerPair = d_propagationMap[toExplain];
+    Trace("theory::explain")
+        << "TheoryEngine::getExplanation: explainer for node "
+        << nodeExplainerPair.d_node
+        << " is theory: " << nodeExplainerPair.d_theory << std::endl;
 
-  NodeTheoryPair nodeExplainerPair = d_propagationMap[toExplain];
-  Trace("theory::explain")
-      << "TheoryEngine::getExplanation: explainer for node "
-      << nodeExplainerPair.d_node
-      << " is theory: " << nodeExplainerPair.d_theory << std::endl;
-
-  // Create the workplace for explanations
-  std::vector<NodeTheoryPair> vec{d_propagationMap[toExplain]};
-  // Process the explanation
-  TrustNode texplanation = getExplanation(vec);
-  Trace("theory::explain") << "TheoryEngine::getExplanation(" << node << ") => "
-                           << texplanation.getNode() << endl;
+    // Create the workplace for explanations
+    std::vector<NodeTheoryPair> vec{d_propagationMap[toExplain]};
+    // Process the explanation
+    texplanation = getExplanation(vec);
+    Trace("theory::explain") << "TheoryEngine::getExplanation(" << node
+                             << ") => " << texplanation.getNode() << endl;
+  }
+  // notify the conflict as a lemma
+  for (TheoryEngineModule* tem : d_modules)
+  {
+    tem->notifyLemma(
+        texplanation.getProven(), LemmaProperty::REMOVABLE, {}, {});
+  }
   return texplanation;
 }
 
@@ -1348,12 +1409,12 @@ void TheoryEngine::lemma(TrustNode tlemma,
     std::vector<Node> sks;
     Node retLemma =
         d_propEngine->getPreprocessedTerm(tlemma.getProven(), skAsserts, sks);
-    if (options().theory.relevanceFilter && isLemmaPropertyNeedsJustify(p))
+
+    // notify the modules of the lemma
+    for (TheoryEngineModule* tem : d_modules)
     {
-      d_relManager->notifyPreprocessedAssertion(retLemma, false);
-      d_relManager->notifyPreprocessedAssertions(skAsserts, false);
+      tem->notifyLemma(retLemma, p, skAsserts, sks);
     }
-    d_relManager->notifyLemma(retLemma);
   }
 
   // Mark that we added some lemmas
@@ -1851,10 +1912,8 @@ void TheoryEngine::checkTheoryAssertionsWithModel(bool hardFailure) {
   bool hasRelevantAssertions = false;
   if (d_relManager != nullptr)
   {
-    d_relManager->beginRound();
     relevantAssertions =
         d_relManager->getRelevantAssertions(hasRelevantAssertions);
-    d_relManager->endRound();
   }
   for(TheoryId theoryId = THEORY_FIRST; theoryId < THEORY_LAST; ++theoryId) {
     Theory* theory = d_theoryTable[theoryId];
