@@ -291,7 +291,7 @@ unsigned HoExtension::checkExtensionality(TheoryModel* m)
     // are present
     if (hasFunctions)
     {
-      d_im.setIncomplete(IncompleteId::UF_HO_EXT_DISABLED);
+      d_im.setModelUnsound(IncompleteId::UF_HO_EXT_DISABLED);
     }
     return 0;
   }
