@@ -52,7 +52,7 @@ TheoryUF::TheoryUF(Env& env,
       d_ho(nullptr),
       d_functionsTerms(context()),
       d_symb(env, instanceName),
-      d_rewriter(logicInfo().isHigherOrder()),
+      d_rewriter(),
       d_state(env, valuation),
       d_im(env, *this, d_state, "theory::uf::" + instanceName, false),
       d_notify(d_im, *this),
@@ -274,7 +274,7 @@ void TheoryUF::preRegisterTerm(TNode node)
   {
     case kind::EQUAL:
       // Add the trigger for equality
-      d_equalityEngine->addTriggerPredicate(node);
+      d_state.addEqualityEngineTriggerPredicate(node);
       break;
     case kind::APPLY_UF:
     case kind::HO_APPLY:
@@ -282,8 +282,7 @@ void TheoryUF::preRegisterTerm(TNode node)
       // Maybe it's a predicate
       if (node.getType().isBoolean())
       {
-        // Get triggered for both equal and dis-equal
-        d_equalityEngine->addTriggerPredicate(node);
+        d_state.addEqualityEngineTriggerPredicate(node);
       }
       else
       {
