@@ -17,11 +17,11 @@
  * system tests.
  */
 
+#include <cvc5/cvc5.h>
+
 #include <iostream>
 #include <sstream>
 
-#include "api/cpp/cvc5.h"
-
 using namespace cvc5;
 using namespace std;
 
