@@ -14,7 +14,7 @@
  *
  */
 
-#include "api/cpp/cvc5.h"
+#include <cvc5/cvc5.h>
 
 using namespace cvc5;
 
