@@ -14,7 +14,8 @@
  *
  */
 
-#include "api/cpp/cvc5.h"
+#include <cvc5/cvc5.h>
+
 #include <cassert>
 
 using namespace cvc5;
