@@ -65,6 +65,23 @@ TEST_F(TestApiBlackSort, operators_comparison)
   ASSERT_NO_THROW(d_solver.getIntegerSort() >= Sort());
 }
 
+TEST_F(TestApiBlackSort, getKind)
+{
+  Sort b = d_solver.getBooleanSort();
+  ASSERT_EQ(b.getKind(), BOOLEAN_SORT);
+  Sort dt_sort = create_datatype_sort();
+  ASSERT_EQ(dt_sort.getKind(), DATATYPE_SORT);
+  Sort arr_sort =
+      d_solver.mkArraySort(d_solver.getRealSort(), d_solver.getIntegerSort());
+  ASSERT_EQ(arr_sort.getKind(), ARRAY_SORT);
+  Sort fp_sort = d_solver.mkFloatingPointSort(8, 24);
+  ASSERT_EQ(fp_sort.getKind(), FLOATINGPOINT_SORT);
+  Sort bv_sort = d_solver.mkBitVectorSort(8);
+  ASSERT_EQ(bv_sort.getKind(), BITVECTOR_SORT);
+  Sort abs_sort = d_solver.mkAbstractSort(BITVECTOR_SORT);
+  ASSERT_EQ(abs_sort.getKind(), ABSTRACT_SORT);
+}
+
 TEST_F(TestApiBlackSort, hasGetSymbol)
 {
   Sort n;
@@ -135,6 +152,12 @@ TEST_F(TestApiBlackSort, isBitVector)
   ASSERT_NO_THROW(Sort().isBitVector());
 }
 
+TEST_F(TestApiBlackSort, isFiniteField)
+{
+  ASSERT_TRUE(d_solver.mkFiniteFieldSort("7").isFiniteField());
+  ASSERT_NO_THROW(Sort().isFiniteField());
+}
+
 TEST_F(TestApiBlackSort, isFloatingPoint)
 {
   ASSERT_TRUE(d_solver.mkFloatingPointSort(8, 24).isFloatingPoint());
@@ -243,6 +266,16 @@ TEST_F(TestApiBlackSort, isSequence)
   ASSERT_NO_THROW(Sort().isSequence());
 }
 
+TEST_F(TestApiBlackSort, isAbstract)
+{
+  ASSERT_TRUE(d_solver.mkAbstractSort(BITVECTOR_SORT).isAbstract());
+  // ?Array is syntax sugar for (Array ? ?), thus the constructed sort
+  // is an Array sort, not an abstract sort.
+  ASSERT_FALSE(d_solver.mkAbstractSort(ARRAY_SORT).isAbstract());
+  ASSERT_TRUE(d_solver.mkAbstractSort(ABSTRACT_SORT).isAbstract());
+  ASSERT_NO_THROW(Sort().isAbstract());
+}
+
 TEST_F(TestApiBlackSort, isUninterpreted)
 {
   Sort un_sort = d_solver.mkUninterpretedSort("asdf");
@@ -486,6 +519,19 @@ TEST_F(TestApiBlackSort, getSequenceElementSort)
   ASSERT_THROW(bvSort.getSequenceElementSort(), CVC5ApiException);
 }
 
+TEST_F(TestApiBlackSort, getAbstractedKind)
+{
+  ASSERT_EQ(d_solver.mkAbstractSort(BITVECTOR_SORT).getAbstractedKind(),
+            BITVECTOR_SORT);
+  // ?Array is syntax sugar for (Array ? ?), thus the constructed sort
+  // is an Array sort, not an abstract sort and its abstract kind cannot be
+  // extracted.
+  ASSERT_THROW(d_solver.mkAbstractSort(ARRAY_SORT).getAbstractedKind(),
+               CVC5ApiException);
+  ASSERT_EQ(d_solver.mkAbstractSort(ABSTRACT_SORT).getAbstractedKind(),
+            ABSTRACT_SORT);
+}
+
 TEST_F(TestApiBlackSort, getSymbol)
 {
   Sort uSort = d_solver.mkUninterpretedSort("u");
@@ -518,6 +564,16 @@ TEST_F(TestApiBlackSort, getBitVectorSize)
   ASSERT_THROW(setSort.getBitVectorSize(), CVC5ApiException);
 }
 
+TEST_F(TestApiBlackSort, getFiniteFieldSize)
+{
+  Sort ffSort = d_solver.mkFiniteFieldSort("31");
+  ASSERT_NO_THROW(ffSort.getFiniteFieldSize());
+  ASSERT_EQ(ffSort.getFiniteFieldSize(), "31");
+  ASSERT_THROW(Sort().getFiniteFieldSize(), CVC5ApiException);
+  Sort setSort = d_solver.mkSetSort(d_solver.getIntegerSort());
+  ASSERT_THROW(setSort.getFiniteFieldSize(), CVC5ApiException);
+}
+
 TEST_F(TestApiBlackSort, getFloatingPointExponentSize)
 {
   Sort fpSort = d_solver.mkFloatingPointSort(4, 8);
