@@ -13,6 +13,7 @@
  * Black box testing of cvc5::parser::ParserBuilder.
  */
 
+#include <cvc5/cvc5.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
@@ -21,11 +22,10 @@
 #include <fstream>
 #include <iostream>
 
-#include "api/cpp/cvc5.h"
 #include "options/language.h"
 #include "parser/api/cpp/command.h"
 #include "parser/api/cpp/symbol_manager.h"
-#include "parser/parser.h"
+#include "parser/parser_antlr.h"
 #include "parser/parser_builder.h"
 #include "test_api.h"
 
@@ -47,10 +47,9 @@ class TestParseBlackParserBuilder : public TestApi
 
   void checkInput(Parser* parser, const std::string& expected)
   {
-    Command* cmd = parser->nextCommand();
+    std::unique_ptr<Command> cmd = parser->nextCommand();
     ASSERT_NE(cmd, nullptr);
     ASSERT_EQ(cmd->toString(), expected);
-    delete cmd;
 
     cmd = parser->nextCommand();
     ASSERT_EQ(cmd, nullptr);
