@@ -16,7 +16,8 @@
 #ifndef CVC5__TEST__UNIT__TEST_API_H
 #define CVC5__TEST__UNIT__TEST_API_H
 
-#include "api/cpp/cvc5.h"
+#include <cvc5/cvc5.h>
+
 #include "gtest/gtest.h"
 
 namespace cvc5::internal {
